import io
import json
import os
import pathlib
import unittest
import tempfile

from robot.errors import DataError
from robot.model.modelobject import ModelObject
from robot.utils import get_error_message
from robot.utils.asserts import assert_equal, assert_raises_with_msg


class Example(ModelObject):

    def __init__(self, a=None, b=None, c=None):
        self.a = a
        self.b = b
        self.c = c

    def __setattr__(self, name, value):
        if value == 'fail':
            raise AttributeError('Ooops!')
        self.__dict__[name] = value

    def to_dict(self):
        return self.__dict__


class TestRepr(unittest.TestCase):

    def test_default(self):
        assert_equal(repr(ModelObject()), 'robot.model.ModelObject()')

    def test_module_when_extending(self):
        assert_equal(repr(Example()), f'{__name__}.Example()')

    def test_repr_args(self):
        class X(ModelObject):
            repr_args = ('z', 'x')
            x, y, z = 1, 2, 3
        assert_equal(repr(X()), f'{__name__}.X(z=3, x=1)')


class TestConfig(unittest.TestCase):

    def test_basics(self):
        x = Example().config(a=1, c=3)
        assert_equal(x.a, 1)
        assert_equal(x.b, None)
        assert_equal(x.c, 3)

    def test_attributes_must_exist(self):
        assert_raises_with_msg(
            AttributeError,
            f"'{__name__}.Example' object does not have attribute 'bad'",
            Example().config, bad='attr'
        )

    def test_setting_attribute_fails(self):
        assert_raises_with_msg(
            AttributeError,
            "Setting attribute 'a' failed: Ooops!",
            Example().config, a='fail'
        )

    def test_preserve_tuples(self):
        x = Example(a=(1, 2, 3)).config(a=range(5))
        assert_equal(x.a, (0, 1, 2, 3, 4))

    def test_failure_converting_to_tuple(self):
        assert_raises_with_msg(
            TypeError,
            f"'{__name__}.Example' object attribute 'a' is 'tuple', got 'None'.",
            Example(a=()).config, a=None
        )


class TestFromDictAndJson(unittest.TestCase):

    def test_attributes(self):
        obj = Example.from_dict({'a': 1})
        assert_equal(obj.a, 1)
        assert_equal(obj.b, None)
        assert_equal(obj.c, None)
        obj = Example.from_json('{"a": null, "b": 42, "c": true}')
        assert_equal(obj.a, None)
        assert_equal(obj.b, 42)
        assert_equal(obj.c, True)

    def test_non_existing_attribute(self):
        assert_raises_with_msg(
            DataError,
            f"Creating '{__name__}.Example' object from dictionary failed: "
            f"'{__name__}.Example' object does not have attribute 'nonex'",
            Example.from_dict, {'nonex': 'attr'}
        )

    def test_setting_attribute_fails(self):
        assert_raises_with_msg(
            DataError,
            f"Creating '{__name__}.Example' object from dictionary failed: "
            f"Setting attribute 'a' failed: Ooops!",
            Example.from_dict, {'a': 'fail'}
        )

    def test_json_as_bytes(self):
        obj = Example.from_json(b'{"a": null, "b": 42}')
        assert_equal(obj.a, None)
        assert_equal(obj.b, 42)

    def test_json_as_open_file(self):
        obj = Example.from_json(io.StringIO('{"a": null, "b": 42, "c": "åäö"}'))
        assert_equal(obj.a, None)
        assert_equal(obj.b, 42)
        assert_equal(obj.c, "åäö")

    def test_json_as_path(self):
        with tempfile.NamedTemporaryFile('w', encoding='UTF-8', delete=False) as file:
            file.write('{"a": null, "b": 42, "c": "åäö"}')
        try:
            for path in file.name, pathlib.Path(file.name):
                obj = Example.from_json(path)
                assert_equal(obj.a, None)
                assert_equal(obj.b, 42)
                assert_equal(obj.c, "åäö")
        finally:
            os.remove(file.name)

    def test_invalid_json_type(self):
        error = self._get_json_load_error(None)
        assert_raises_with_msg(
            DataError,
            f"Loading JSON data failed: Invalid JSON data: {error}",
            ModelObject.from_json, None
        )

    def test_invalid_json_syntax(self):
        error = self._get_json_load_error('bad')
        assert_raises_with_msg(
            DataError,
            f"Loading JSON data failed: Invalid JSON data: {error}",
            ModelObject.from_json, 'bad'
        )

    def test_invalid_json_content(self):
        assert_raises_with_msg(
            DataError,
            "Loading JSON data failed: Expected dictionary, got list.",
            ModelObject.from_json, '["bad"]'
        )

    def _get_json_load_error(self, value):
        try:
            json.loads(value)
        except Exception:
            return get_error_message()


class TestToJson(unittest.TestCase):
    data = {'a': 1, 'b': [True, False], 'c': 'nön-äscii'}
    default_config = {'ensure_ascii': False, 'indent': 0, 'separators': (',', ':')}
    custom_config = {'indent': None, 'separators': (', ', ': '), 'ensure_ascii': True}

    def test_default_config(self):
        assert_equal(Example(**self.data).to_json(),
                     json.dumps(self.data, **self.default_config))

    def test_custom_config(self):
        assert_equal(Example(**self.data).to_json(**self.custom_config),
                     json.dumps(self.data, **self.custom_config))

    def test_write_to_open_file(self):
        for config in {}, self.custom_config:
            output = io.StringIO()
            Example(**self.data).to_json(output, **config)
            expected = json.dumps(self.data, **(config or self.default_config))
            assert_equal(output.getvalue(), expected)

    def test_write_to_path(self):
        with tempfile.NamedTemporaryFile(delete=False) as file:
            pass
        try:
            for path in file.name, pathlib.Path(file.name):
                for config in {}, self.custom_config:
                    Example(**self.data).to_json(path, **config)
                    expected = json.dumps(self.data, **(config or self.default_config))
                    with open(path, encoding='UTF-8') as file:
                        assert_equal(file.read(), expected)
        finally:
            os.remove(file.name)

    def test_invalid_output(self):
        assert_raises_with_msg(TypeError,
                               "Output should be None, path or open file, got integer.",
                               Example().to_json, 42)


if __name__ == '__main__':
    unittest.main()



import logging
import time

from robot.api import logger
from robot.output.pyloggingconf import RobotHandler


# Use simpler formatter to avoid flakeynes that started to occur after enhancing
# message formatting in https://github.com/robotframework/robotframework/pull/4147
# Without this change execution on PyPy failed about every third time so that
# timeout was somehow ignored. On CI the problem occurred also with Python 3.9.
# Not sure why the problem occurred but it seems to be related to the logging
# module and not related to the bug that this library is testing. This hack ought
# ought to thus be safe. With it was able to run tests locally 100 times using
# PyPy without problems.
for handler in logging.getLogger().handlers:
    if isinstance(handler, RobotHandler):
        handler.format = lambda record: record.getMessage()


MSG = 'A rather long message that is slow to write on the disk. ' * 10000


def rf_logger():
    _log_a_lot(logger.info)


def python_logger():
    _log_a_lot(logging.info)


def _log_a_lot(info):
    # Assigning local variables is performance optimization to give as much
    # time as as possible for actual logging.
    msg = MSG
    sleep = time.sleep
    current = time.time
    end = current() + 1
    while current() < end:
        info(msg)
        sleep(0)    # give time for other threads
    raise AssertionError('Execution should have been stopped by timeout.')



from robot.libraries.BuiltIn import BuiltIn


class ParameterLibrary:

    def __init__(self, host='localhost', port='8080'):
        self.host = host
        self.port = port

    def parameters(self):
        return self.host, self.port

    def parameters_should_be(self, host='localhost', port='8080'):
        should_be_equal = BuiltIn().should_be_equal
        should_be_equal(self.host, host)
        should_be_equal(self.port, port)


class V1(ParameterLibrary): pass
class V2(ParameterLibrary): pass
class V3(ParameterLibrary): pass
class V4(ParameterLibrary): pass
class V5(ParameterLibrary): pass
class V6(ParameterLibrary): pass



import sys
import unittest

from robot.errors import DataError
from robot.model import Body
from robot.running.userkeyword import EmbeddedArgumentsHandler
from robot.running.arguments import EmbeddedArguments, UserKeywordArgumentParser
from robot.utils.asserts import (assert_equal, assert_true, assert_raises,
                                 assert_raises_with_msg)


class Fake:
    value = ''
    message = ''

    def __iter__(self):
        return iter([])


class FakeArgs:

    def __init__(self, args):
        self.value = args

    def __nonzero__(self):
        return bool(self.value)

    def __iter__(self):
        return iter(self.value)


class HandlerDataMock:

    def __init__(self, name, args=[]):
        self.name = name
        self.args = FakeArgs(args)
        self.body = Body()
        self.source = None
        self.lineno = -1
        self.return_value = None
        self.doc = Fake()
        self.timeout = Fake()
        self.return_ = Fake()
        self.tags = ()
        self.has_teardown = False


def EAT(name, args=[]):
    handler = HandlerDataMock(name, args)
    embedded = EmbeddedArguments.from_name(name)
    return EmbeddedArgumentsHandler(handler, 'resource', embedded)


class TestEmbeddedArgs(unittest.TestCase):

    def setUp(self):
        self.tmp1 = EAT('User selects ${item} from list')
        self.tmp2 = EAT('${x} * ${y} from "${z}"')

    def test_truthy(self):
        assert_true(EmbeddedArguments.from_name('${Yes} embedded args here'))
        assert_true(not EmbeddedArguments.from_name('No embedded args here'))

    def test_get_embedded_arg_and_regexp(self):
        assert_equal(self.tmp1.embedded.args, ['item'])
        assert_equal(self.tmp1.embedded.name.pattern,
                     '^User\\ selects\\ (.*?)\\ from\\ list$')
        assert_equal(self.tmp1.name, 'User selects ${item} from list')

    def test_get_multiple_embedded_args_and_regexp(self):
        assert_equal(self.tmp2.embedded.args, ['x', 'y', 'z'])
        quote = '"' if sys.version_info[:2] >= (3, 7) else '\\"'
        assert_equal(self.tmp2.embedded.name.pattern,
                     '^(.*?)\\ \\*\\ (.*?)\\ from\\ {0}(.*?){0}$'.format(quote))

    def test_create_runner_with_one_embedded_arg(self):
        runner = self.tmp1.create_runner('User selects book from list')
        assert_equal(runner.embedded_args, ('book',))
        assert_equal(runner.name, 'User selects book from list')
        assert_equal(runner.longname, 'resource.User selects book from list')
        runner = self.tmp1.create_runner('User selects radio from list')
        assert_equal(runner.embedded_args, ('radio',))
        assert_equal(runner.name, 'User selects radio from list')
        assert_equal(runner.longname, 'resource.User selects radio from list')

    def test_create_runner_with_many_embedded_args(self):
        runner = self.tmp2.create_runner('User * book from "list"')
        assert_equal(runner.embedded_args, ('User', 'book', 'list'))

    def test_create_runner_with_empty_embedded_arg(self):
        runner = self.tmp1.create_runner('User selects  from list')
        assert_equal(runner.embedded_args, ('',))

    def test_create_runner_with_special_characters_in_embedded_args(self):
        runner = self.tmp2.create_runner('Janne & Heikki * "enjoy" from """')
        assert_equal(runner.embedded_args, ('Janne & Heikki', '"enjoy"', '"'))

    def test_embedded_args_without_separators(self):
        template = EAT('This ${does}${not} work so well')
        runner = template.create_runner('This doesnot work so well')
        assert_equal(runner.embedded_args, ('', 'doesnot'))

    def test_embedded_args_with_separators_in_values(self):
        template = EAT('This ${could} ${work}-${OK}')
        runner = template.create_runner("This doesn't really work---")
        assert_equal(runner.embedded_args, ("doesn't", 'really work', '--'))

    def test_creating_runners_is_case_insensitive(self):
        runner = self.tmp1.create_runner('User SELECts book frOm liST')
        assert_equal(runner.embedded_args, ('book',))
        assert_equal(runner.name, 'User SELECts book frOm liST')
        assert_equal(runner.longname, 'resource.User SELECts book frOm liST')


class TestGetArgSpec(unittest.TestCase):

    def test_no_args(self):
        self._verify('')

    def test_args(self):
        self._verify('${arg1}', ['arg1',])
        self._verify('${a1} ${a2}', ['a1', 'a2'])

    def test_defaults(self):
        self._verify('${arg1} ${arg2}=default @{varargs}',
                     args=['arg1', 'arg2'],
                     defaults={'arg2': 'default'},
                     varargs='varargs')
        self._verify('${arg1} ${arg2}= @{varargs}',
                     args=['arg1', 'arg2'],
                     defaults={'arg2': ''},
                     varargs='varargs')
        self._verify('${arg1}=d1 ${arg2}=d2 ${arg3}=d3',
                     args=['arg1', 'arg2', 'arg3'],
                     defaults={'arg1': 'd1', 'arg2': 'd2', 'arg3': 'd3'})

    def test_vararg(self):
        self._verify('@{varargs}', varargs='varargs')
        self._verify('${arg} @{varargs}', ['arg'], varargs='varargs')

    def test_kwonly(self):
        self._verify('@{} ${ko1} ${ko2}',
                     kwonlyargs=['ko1', 'ko2'])
        self._verify('@{vars} ${ko1} ${ko2}',
                     varargs='vars',
                     kwonlyargs=['ko1', 'ko2'])

    def test_kwonlydefaults(self):
        self._verify('@{} ${ko1} ${ko2}=xxx',
                     kwonlyargs=['ko1', 'ko2'],
                     defaults={'ko2': 'xxx'})
        self._verify('@{} ${ko1}=xxx ${ko2}',
                     kwonlyargs=['ko1', 'ko2'],
                     defaults={'ko1': 'xxx'})
        self._verify('@{v} ${ko1}=foo ${ko2} ${ko3}=',
                     varargs='v',
                     kwonlyargs=['ko1', 'ko2', 'ko3'],
                     defaults={'ko1': 'foo', 'ko3': ''})

    def test_kwargs(self):
        self._verify('&{kwargs}', kwargs='kwargs')
        self._verify('${arg} &{kwargs}',
                     args=['arg'],
                     kwargs='kwargs')
        self._verify('@{} ${arg} &{kwargs}',
                     kwonlyargs=['arg'],
                     kwargs='kwargs')
        self._verify('${a1} ${a2}=ad @{vars} ${k1} ${k2}=kd &{kws}',
                     args=['a1', 'a2'],
                     varargs='vars',
                     kwonlyargs=['k1', 'k2'],
                     defaults={'a2': 'ad', 'k2': 'kd'},
                     kwargs='kws')

    def _verify(self, in_args, args=[], defaults={}, varargs=None,
                kwonlyargs=[], kwargs=None):
        argspec = self._parse(in_args)
        assert_equal(argspec.positional, args)
        assert_equal(argspec.defaults, defaults)
        assert_equal(argspec.var_positional, varargs)
        assert_equal(argspec.named_only, kwonlyargs)
        assert_equal(argspec.var_named, kwargs)

    def _parse(self, in_args):
        return UserKeywordArgumentParser().parse(in_args.split())

    def test_arg_after_defaults(self):
        self._verify_error('${arg1}=default ${arg2}',
                           'Non-default argument after default arguments.')

    def test_multiple_varargs(self):
        for spec in ['@{v1} @{v2}', '@{} @{v}', '@{v} @{}', '@{} @{}']:
            self._verify_error(spec, 'Cannot have multiple varargs.')

    def test_args_after_kwargs(self):
        self._verify_error('&{kws} ${arg}',
                           'Only last argument can be kwargs.')

    def _verify_error(self, in_args, exp_error):
        assert_raises_with_msg(DataError,
                               'Invalid argument specification: ' + exp_error,
                               self._parse, in_args)


if __name__ == '__main__':
    unittest.main()



import unittest

from robot.utils.asserts import assert_equal
from robot.model.statistics import Statistics
from robot.result import TestCase, TestSuite


def verify_stat(stat, name, passed, failed, skipped,
                combined=None, id=None, elapsed=0):
    assert_equal(stat.name, name, 'stat.name')
    assert_equal(stat.passed, passed)
    assert_equal(stat.failed, failed)
    assert_equal(stat.skipped, skipped)
    assert_equal(stat.total, passed + failed + skipped)
    if hasattr(stat, 'combined'):
        assert_equal(stat.combined, combined)
    if hasattr(stat, 'id'):
        assert_equal(stat.id, id)
    assert_equal(stat.elapsed, elapsed)


def verify_suite(suite, name, id, passed, failed, skipped):
    verify_stat(suite.stat, name, passed, failed, skipped, id=id)


def generate_suite():
    suite = TestSuite(name='Root Suite')
    s1 = suite.suites.create(name='First Sub Suite')
    s2 = suite.suites.create(name='Second Sub Suite')
    s11 = s1.suites.create(name='Sub Suite 1_1')
    s12 = s1.suites.create(name='Sub Suite 1_2')
    s13 = s1.suites.create(name='Sub Suite 1_3')
    s21 = s2.suites.create(name='Sub Suite 2_1')
    s22 = s2.suites.create(name='Sub Suite 3_1')
    s11.tests = [TestCase(status='PASS'), TestCase(status='FAIL', tags=['t1'])]
    s12.tests = [TestCase(status='PASS', tags=['t_1','t2',]),
                 TestCase(status='PASS', tags=['t1','smoke']),
                 TestCase(status='SKIP', tags=['t1','flaky']),
                 TestCase(status='FAIL', tags=['t1','t2','t3','smoke'])]
    s13.tests = [TestCase(status='PASS', tags=['t1','t 2','smoke'])]
    s21.tests = [TestCase(status='FAIL', tags=['t3','Smoke'])]
    s22.tests = [TestCase(status='SKIP', tags=['flaky'])]
    return suite


class TestStatisticsSimple(unittest.TestCase):

    def setUp(self):
        suite = TestSuite(name='Hello')
        suite.tests = [TestCase(status='PASS'), TestCase(status='PASS'),
                       TestCase(status='FAIL'), TestCase(status='SKIP')]
        self.statistics = Statistics(suite)

    def test_total(self):
        verify_stat(self.statistics.total._stat, 'All Tests', 2, 1, 1)

    def test_suite(self):
        verify_suite(self.statistics.suite, 'Hello', 's1', 2, 1, 1)

    def test_tags(self):
        assert_equal(list(self.statistics.tags), [])


class TestStatisticsNotSoSimple(unittest.TestCase):

    def setUp(self):
        suite = generate_suite()
        self.statistics = Statistics(suite, 2, ['t*','smoke'], ['t3'],
                                     [('t? & smoke', ''), ('none NOT t1', 'a title')])

    def test_total(self):
        verify_stat(self.statistics.total._stat, 'All Tests', 4, 3, 2)

    def test_suite(self):
        suite = self.statistics.suite
        verify_suite(suite, 'Root Suite', 's1', 4, 3,2 )
        [s1, s2] = suite.suites
        verify_suite(s1, 'Root Suite.First Sub Suite', 's1-s1', 4, 2, 1)
        verify_suite(s2, 'Root Suite.Second Sub Suite', 's1-s2', 0, 1, 1)
        assert_equal(len(s1.suites), 0)
        assert_equal(len(s2.suites), 0)

    def test_tags(self):
        # Tag stats are tested more thoroughly in test_tagstatistics.py
        tags = self.statistics.tags
        verify_stat(tags.tags['smoke'], 'smoke', 2, 2, 0)
        verify_stat(tags.tags['t1'], 't1', 3, 2, 1)
        verify_stat(tags.tags['t2'], 't2', 2, 1, 0)
        expected = [('a title', 0, 0, 0, 'none NOT t1'),
                    ('t? & smoke', 2, 2, 0, 't? & smoke'),
                    ('smoke', 2, 2, 0),
                    ('t1', 3, 2, 1),
                    ('t2', 2, 1, 0)]
        assert_equal(len(list(tags)), len(expected))
        for t, e in zip(tags, expected):
            verify_stat(t, *e)


class TestSuiteStatistics(unittest.TestCase):

    def test_all_levels(self):
        suite = Statistics(generate_suite()).suite
        verify_suite(suite, 'Root Suite', 's1', 4, 3, 2)
        [s1, s2] = suite.suites
        verify_suite(s1, 'Root Suite.First Sub Suite', 's1-s1', 4, 2, 1)
        verify_suite(s2, 'Root Suite.Second Sub Suite', 's1-s2', 0, 1, 1)
        [s11, s12, s13] = s1.suites
        verify_suite(s11, 'Root Suite.First Sub Suite.Sub Suite 1_1', 's1-s1-s1', 1, 1, 0)
        verify_suite(s12, 'Root Suite.First Sub Suite.Sub Suite 1_2', 's1-s1-s2', 2, 1, 1)
        verify_suite(s13, 'Root Suite.First Sub Suite.Sub Suite 1_3', 's1-s1-s3', 1, 0, 0)
        [s21, s22] = s2.suites
        verify_suite(s21, 'Root Suite.Second Sub Suite.Sub Suite 2_1', 's1-s2-s1', 0, 1, 0)
        verify_suite(s22, 'Root Suite.Second Sub Suite.Sub Suite 3_1', 's1-s2-s2', 0, 0, 1)

    def test_only_root_level(self):
        suite = Statistics(generate_suite(), suite_stat_level=1).suite
        verify_suite(suite, 'Root Suite', 's1', 4, 3, 2)
        assert_equal(len(suite.suites), 0)

    def test_deeper_level(self):
        PASS = TestCase(status='PASS')
        FAIL = TestCase(status='FAIL')
        SKIP = TestCase(status='SKIP')
        suite = TestSuite(name='1')
        suite.suites = [TestSuite(name='1'), TestSuite(name='2'), TestSuite(name='3')]
        suite.suites[0].suites = [TestSuite(name='1')]
        suite.suites[1].suites = [TestSuite(name='1'), TestSuite(name='2')]
        suite.suites[2].tests = [PASS, FAIL]
        suite.suites[0].suites[0].suites = [TestSuite(name='1')]
        suite.suites[1].suites[0].tests = [PASS, PASS, PASS, FAIL, SKIP]
        suite.suites[1].suites[1].tests = [PASS, PASS, FAIL, SKIP]
        suite.suites[0].suites[0].suites[0].tests = [FAIL, FAIL, FAIL]
        s1 = Statistics(suite, suite_stat_level=3).suite
        verify_suite(s1, '1', 's1', 6, 6, 2)
        [s11, s12, s13] = s1.suites
        verify_suite(s11, '1.1', 's1-s1', 0, 3, 0)
        verify_suite(s12, '1.2', 's1-s2', 5, 2, 2)
        verify_suite(s13, '1.3', 's1-s3', 1, 1, 0)
        [s111] = s11.suites
        verify_suite(s111, '1.1.1', 's1-s1-s1', 0, 3, 0)
        [s121, s122] = s12.suites
        verify_suite(s121, '1.2.1', 's1-s2-s1', 3, 1, 1)
        verify_suite(s122, '1.2.2', 's1-s2-s2', 2, 1, 1)
        assert_equal(len(s111.suites), 0)

    def test_iter_only_one_level(self):
        [stat] = list(Statistics(generate_suite(), suite_stat_level=1).suite)
        verify_stat(stat, 'Root Suite', 4, 3, 2, id='s1')

    def test_iter_also_sub_suites(self):
        stats = list(Statistics(generate_suite()).suite)
        verify_stat(stats[0], 'Root Suite', 4, 3, 2, id='s1')
        verify_stat(stats[1], 'Root Suite.First Sub Suite', 4, 2, 1, id='s1-s1')
        verify_stat(stats[2], 'Root Suite.First Sub Suite.Sub Suite 1_1', 1, 1, 0, id='s1-s1-s1')
        verify_stat(stats[3], 'Root Suite.First Sub Suite.Sub Suite 1_2', 2, 1, 1, id='s1-s1-s2')
        verify_stat(stats[4], 'Root Suite.First Sub Suite.Sub Suite 1_3', 1, 0, 0, id='s1-s1-s3')
        verify_stat(stats[5], 'Root Suite.Second Sub Suite', 0, 1, 1, id='s1-s2')
        verify_stat(stats[6], 'Root Suite.Second Sub Suite.Sub Suite 2_1', 0, 1, 0, id='s1-s2-s1')
        verify_stat(stats[7], 'Root Suite.Second Sub Suite.Sub Suite 3_1', 0, 0, 1, id='s1-s2-s2')


class TestElapsedTime(unittest.TestCase):

    def setUp(self):
        ts = '20120816 00:00:'
        suite = TestSuite(starttime=ts+'00.000', endtime=ts+'59.999')
        suite.suites = [
            TestSuite(starttime=ts+'00.000', endtime=ts+'30.000'),
            TestSuite(starttime=ts+'30.000', endtime=ts+'42.042')
        ]
        suite.suites[0].tests = [
            TestCase(starttime=ts+'00.000', endtime=ts+'00.001', tags=['t1']),
            TestCase(starttime=ts+'00.001', endtime=ts+'01.001', tags=['t1', 't2'])
        ]
        suite.suites[1].tests = [
            TestCase(starttime=ts+'30.000', endtime=ts+'40.000', tags=['t1', 't2', 't3'])
        ]
        self.stats = Statistics(suite, tag_stat_combine=[('?2', 'combined')])

    def test_total_stats(self):
        assert_equal(self.stats.total._stat.elapsed, 11001)

    def test_tag_stats(self):
        t1, t2, t3 = self.stats.tags.tags.values()
        verify_stat(t1, 't1', 0, 3, 0, elapsed=11001)
        verify_stat(t2, 't2', 0, 2, 0, elapsed=11000)
        verify_stat(t3, 't3', 0, 1, 0, elapsed=10000)

    def test_combined_tag_stats(self):
        combined = self.stats.tags.combined[0]
        verify_stat(combined, 'combined', 0, 2, 0, combined='?2', elapsed=11000)

    def test_suite_stats(self):
        assert_equal(self.stats.suite.stat.elapsed, 59999)
        assert_equal(self.stats.suite.suites[0].stat.elapsed, 30000)
        assert_equal(self.stats.suite.suites[1].stat.elapsed, 12042)

    def test_suite_stats_when_suite_has_no_times(self):
        suite = TestSuite()
        assert_equal(Statistics(suite).suite.stat.elapsed, 0)
        ts = '20120816 00:00:'
        suite.tests = [TestCase(starttime=ts+'00.000', endtime=ts+'00.001'),
                       TestCase(starttime=ts+'00.001', endtime=ts+'01.001')]
        assert_equal(Statistics(suite).suite.stat.elapsed, 1001)
        suite.suites = [TestSuite(starttime=ts+'02.000', endtime=ts+'12.000'),
                        TestSuite()]
        assert_equal(Statistics(suite).suite.stat.elapsed, 11001)

    def test_elapsed_from_get_attributes(self):
        for time, expected in [('00:00:00.000', '00:00:00'),
                               ('00:00:00.001', '00:00:00'),
                               ('00:00:00.500', '00:00:00'),
                               ('00:00:00.501', '00:00:01'),
                               ('00:00:00.999', '00:00:01'),
                               ('00:00:01.000', '00:00:01'),
                               ('00:00:01.001', '00:00:01'),
                               ('00:00:01.499', '00:00:01'),
                               ('00:00:01.500', '00:00:02'),
                               ('01:59:59:499', '01:59:59'),
                               ('01:59:59:500', '02:00:00')]:
            suite = TestSuite(starttime='20120817 00:00:00.000',
                              endtime='20120817 ' + time)
            stat = Statistics(suite).suite.stat
            elapsed = stat.get_attributes(include_elapsed=True)['elapsed']
            assert_equal(elapsed, expected, time)


if __name__ == "__main__":
    unittest.main()



from io import StringIO
import sys
import tempfile
import unittest

from robot import libdoc
from robot.utils.asserts import assert_equal


class TestLibdoc(unittest.TestCase):

    def setUp(self):
        sys.stdout = StringIO()

    def tearDown(self):
        sys.stdout = sys.__stdout__

    def test_html(self):
        output = tempfile.mkstemp(suffix='.html')[1]
        libdoc.libdoc('String', output)
        assert_equal(sys.stdout.getvalue().strip(), output)
        with open(output) as f:
            assert '"name": "String"' in f.read()

    def test_xml(self):
        output = tempfile.mkstemp(suffix='.xml')[1]
        libdoc.libdoc('String', output)
        assert_equal(sys.stdout.getvalue().strip(), output)
        with open(output) as f:
            assert 'name="String"' in f.read()

    def test_format(self):
        output = tempfile.mkstemp()[1]
        libdoc.libdoc('String', output, format='xml')
        assert_equal(sys.stdout.getvalue().strip(), output)
        with open(output) as f:
            assert 'name="String"' in f.read()

    def test_quiet(self):
        output = tempfile.mkstemp(suffix='.html')[1]
        libdoc.libdoc('String', output, quiet=True)
        assert_equal(sys.stdout.getvalue().strip(), '')
        with open(output) as f:
            assert '"name": "String"' in f.read()

    def test_LibraryDocumentation(self):
        doc = libdoc.LibraryDocumentation('OperatingSystem')
        assert_equal(doc.name, 'OperatingSystem')


if __name__ == '__main__':
    unittest.main()



def get_variables():
    return {'MIXED USAGE': MixedUsage()}


class MixedUsage:

    def __init__(self):
        self.data = ['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K']

    def __getitem__(self, item):
        if isinstance(item, slice) and item.start is item.stop is item.step is None:
            return self
        if isinstance(item, (int, slice)):
            return self.data[item]
        if isinstance(item, str):
            return self.data.index(item)
        raise TypeError



import unittest
import os

from robot.running import userkeyword
from robot.running.model import ResourceFile, UserKeyword
from robot.running.userkeyword import EmbeddedArguments, UserLibrary
from robot.utils.asserts import (assert_equal, assert_none,
                                 assert_raises_with_msg, assert_true)


class UserHandlerStub:

    def __init__(self, kwdata, library):
        self.name = kwdata.name
        self.libname = library
        self.lineno = 42
        if kwdata.name == 'FAIL':
            raise Exception('Expected failure')

    def create(self, name):
        return self


class EmbeddedArgsHandlerStub:

    def __init__(self, kwdata, library, embedded):
        if '${' not in kwdata.name:
            raise TypeError
        self.name = kwdata.name
        self.embedded = embedded

    def matches(self, name):
        return self.embedded.match(name)


class TestUserLibrary(unittest.TestCase):

    def setUp(self):
        self._orig_user_handler = userkeyword.UserKeywordHandler
        self._orig_embedded_handler = userkeyword.EmbeddedArgumentsHandler
        userkeyword.UserKeywordHandler = UserHandlerStub
        userkeyword.EmbeddedArgumentsHandler = EmbeddedArgsHandlerStub

    def tearDown(self):
        userkeyword.UserKeywordHandler = self._orig_user_handler
        userkeyword.EmbeddedArgumentsHandler = self._orig_embedded_handler

    def test_name_from_resource(self):
        for source, exp in [('resources.html', 'resources'),
                            (os.path.join('..','res','My Res.HTM'), 'My Res'),
                            (os.path.abspath('my_res.xhtml'), 'my_res')]:
            lib = self._get_userlibrary(source=source)
            assert_equal(lib.name, exp)

    def test_name_from_test_case_file(self):
        assert_none(self._get_userlibrary().name)

    def test_creating_keyword(self):
        lib = self._get_userlibrary('kw 1', 'kw 2')
        assert_equal(len(lib.handlers), 2)
        assert_true('kw 1' in lib.handlers)
        assert_true('kw 2' in lib.handlers)

    def test_creating_keyword_when_kw_name_has_embedded_arg(self):
        lib = self._get_userlibrary('Embedded ${arg}')
        self._lib_has_embedded_arg_keyword(lib)

    def test_creating_keywords_when_normal_and_embedded_arg_kws(self):
        lib = self._get_userlibrary('kw1', 'Embedded ${arg}', 'kw2')
        assert_equal(len(lib.handlers), 3)
        assert_true('kw1' in lib.handlers)
        assert_true('kw 2' in lib.handlers)
        self._lib_has_embedded_arg_keyword(lib)

    def test_creating_duplicate_embedded_arg_keyword_in_resource_file(self):
        lib = self._get_userlibrary('Embedded ${arg}', 'kw', 'Embedded ${arg}')
        assert_equal(len(lib.handlers), 3)
        assert_true(not hasattr(lib.handlers['kw'], 'error'))
        self._lib_has_embedded_arg_keyword(lib, count=2)

    def test_creating_duplicate_keyword_in_resource_file(self):
        lib = self._get_userlibrary('kw', 'kw', 'kw 2')
        assert_equal(len(lib.handlers), 2)
        assert_true('kw' in lib.handlers)
        assert_true('kw 2' in lib.handlers)
        assert_equal(lib.handlers['kw'].error.message,
                     "Keyword with same name defined multiple times.")

    def test_creating_duplicate_keyword_in_test_case_file(self):
        lib = self._get_userlibrary('MYKW', 'my kw')
        assert_equal(len(lib.handlers), 1)
        assert_true('mykw' in lib.handlers)
        assert_equal(lib.handlers['mykw'].error.message,
                     "Keyword with same name defined multiple times.")

    def test_handlers_contains(self):
        lib = self._get_userlibrary('kw')
        assert_true('kw' in lib.handlers)
        assert_true('nonex' not in lib.handlers)

    def test_handlers_getitem_with_non_existing_keyword(self):
        lib = self._get_userlibrary('kw')
        assert_raises_with_msg(
            ValueError, "No handler with name 'non existing' found.",
            lib.handlers.__getitem__, 'non existing'
        )

    def test_handlers_getitem_with_multiple_matching_keywords(self):
        lib = self._get_userlibrary('Embedded ${a}', 'Embedded ${b}')
        assert_raises_with_msg(
            ValueError,
            "Multiple handlers matching name 'Embedded x' found: 'Embedded ${a}' and 'Embedded ${b}'",
            lib.handlers.__getitem__, 'Embedded x'
        )

    def test_handlers_getitem_with_existing_keyword(self):
        lib = self._get_userlibrary('kw')
        handler = lib.handlers['kw']
        assert_true(isinstance(handler, UserHandlerStub))

    def _get_userlibrary(self, *keywords, **conf):
        resource = ResourceFile(**conf)
        resource.keywords = [UserKeyword(name) for name in keywords]
        return UserLibrary(resource, resource_file='source' in conf)

    def _lib_has_embedded_arg_keyword(self, lib, count=1):
        assert_true('Embedded ${arg}' in lib.handlers)
        embedded = lib.handlers._embedded
        assert_equal(len(embedded), count)
        for template in embedded:
            assert_equal(template.name, 'Embedded ${arg}')


if __name__ == '__main__':
    unittest.main()



import unittest
import re

from robot.utils import safe_str, prepr, DotDict
from robot.utils.asserts import assert_equal, assert_true


class TestSafeStr(unittest.TestCase):

    def test_unicode_nfc_and_nfd_decomposition_equality(self):
        import unicodedata
        text = 'Hyv\xe4'
        assert_equal(safe_str(unicodedata.normalize('NFC', text)), text)
        # In Mac filesystem umlaut characters are presented in NFD-format.
        # This is to check that unic normalizes all strings to NFC
        assert_equal(safe_str(unicodedata.normalize('NFD', text)), text)

    def test_object_containing_unicode_repr(self):
        assert_equal(safe_str(UnicodeRepr()), 'Hyv\xe4')

    def test_list_with_objects_containing_unicode_repr(self):
        objects = [UnicodeRepr(), UnicodeRepr()]
        result = safe_str(objects)
        assert_equal(result, '[Hyv\xe4, Hyv\xe4]')

    def test_bytes_below_128(self):
        assert_equal(safe_str('\x00-\x01-\x02-\x7f'), '\x00-\x01-\x02-\x7f')

    def test_bytes_above_128(self):
        assert_equal(safe_str(b'hyv\xe4'), 'hyv\\xe4')
        assert_equal(safe_str(b'\x00-\x01-\x02-\xe4'), '\x00-\x01-\x02-\\xe4')

    def test_bytes_with_newlines_tabs_etc(self):
        assert_equal(safe_str(b"\x00\xe4\n\t\r\\'"), "\x00\\xe4\n\t\r\\'")

    def test_bytearray(self):
        assert_equal(safe_str(bytearray(b'hyv\xe4')), 'hyv\\xe4')
        assert_equal(safe_str(bytearray(b'\x00-\x01-\x02-\xe4')), '\x00-\x01-\x02-\\xe4')
        assert_equal(safe_str(bytearray(b"\x00\xe4\n\t\r\\'")), "\x00\\xe4\n\t\r\\'")

    def test_failure_in_str(self):
        failing = StrFails()
        assert_equal(safe_str(failing), failing.unrepr)


class TestPrettyRepr(unittest.TestCase):

    def _verify(self, item, expected=None, **config):
        if not expected:
            expected = repr(item).lstrip('')
        assert_equal(prepr(item, **config), expected)
        if isinstance(item, (str, bytes)) and not config:
            assert_equal(prepr([item]), '[%s]' % expected)
            assert_equal(prepr((item,)), '(%s,)' % expected)
            assert_equal(prepr({item: item}), '{%s: %s}' % (expected, expected))
            assert_equal(prepr({item}), '{%s}' % expected)

    def test_ascii_unicode(self):
        self._verify('foo', "'foo'")
        self._verify("f'o'o", "\"f'o'o\"")

    def test_non_ascii_unicode(self):
        self._verify('hyv\xe4', "'hyv\xe4'")

    def test_unicode_in_nfd(self):
        self._verify('hyva\u0308', "'hyv\xe4'")

    def test_ascii_bytes(self):
        self._verify(b'ascii', "b'ascii'")

    def test_non_ascii_bytes(self):
        self._verify(b'non-\xe4scii', "b'non-\\xe4scii'")

    def test_bytearray(self):
        self._verify(bytearray(b'foo'), "bytearray(b'foo')")

    def test_non_strings(self):
        for inp in [1, -2.0, True, None, -2.0, (), [], {}, StrFails()]:
            self._verify(inp)

    def test_failing_repr(self):
        failing = ReprFails()
        self._verify(failing, failing.unrepr)

    def test_unicode_repr(self):
        obj = UnicodeRepr()
        self._verify(obj, 'Hyv\xe4')

    def test_bytes_repr(self):
        obj = BytesRepr()
        self._verify(obj, obj.unrepr)

    def test_collections(self):
        self._verify(['foo', b'bar', 3], "['foo', b'bar', 3]")
        self._verify(['foo', b'b\xe4r', ('x', b'y')], "['foo', b'b\\xe4r', ('x', b'y')]")
        self._verify({'x': b'\xe4'}, "{'x': b'\\xe4'}")
        self._verify(['\xe4'], "['\xe4']")
        self._verify({'\xe4'}, "{'\xe4'}")

    def test_dotdict(self):
        self._verify(DotDict({'x': b'\xe4'}), "{'x': b'\\xe4'}")

    def test_recursive(self):
        x = [1, 2]
        x.append(x)
        match = re.match(r'\[1, 2. <Recursion on list with id=\d+>]', prepr(x))
        assert_true(match is not None)

    def test_split_big_collections(self):
        self._verify(list(range(20)))
        self._verify(list(range(100)), width=400)
        self._verify(list(range(100)),
                     '[%s]' % ',\n '.join(str(i) for i in range(100)))
        self._verify(['Hello, world!'] * 4,
                     '[%s]' % ', '.join(["'Hello, world!'"] * 4))
        self._verify(['Hello, world!'] * 25,
                     '[%s]' % ', '.join(["'Hello, world!'"] * 25), width=500)
        self._verify(['Hello, world!'] * 25,
                     '[%s]' % ',\n '.join(["'Hello, world!'"] * 25))

    def test_dont_split_long_strings(self):
        self._verify(' '.join(['Hello world!'] * 1000))
        self._verify(b' '.join([b'Hello world!'] * 1000),
                     "b'%s'" % ' '.join(['Hello world!'] * 1000))
        self._verify(bytearray(b' '.join([b'Hello world!'] * 1000)))


class UnRepr:
    error = 'This, of course, should never happen...'

    @property
    def unrepr(self):
        return self.format(type(self).__name__, self.error)

    @staticmethod
    def format(name, error):
        return "<Unrepresentable object %s. Error: %s>" % (name, error)


class StrFails(UnRepr):
    def __str__(self):
        raise RuntimeError(self.error)


class ReprFails(UnRepr):
    def __repr__(self):
        raise RuntimeError(self.error)


class UnicodeRepr(UnRepr):

    def __init__(self):
        try:
            repr(self)
        except UnicodeEncodeError as err:
            self.error = 'UnicodeEncodeError: %s' % err

    def __repr__(self):
        return 'Hyv\xe4'


class BytesRepr(UnRepr):

    def __init__(self):
        try:
            repr(self)
        except TypeError as err:
            self.error = 'TypeError: %s' % err

    def __repr__(self):
        return b'Hyv\xe4'


if __name__ == '__main__':
    unittest.main()



import unittest
from robot.utils.asserts import assert_equal, assert_raises

from robot.utils import setter, SetterAwareType


class ExampleWithSlots(metaclass=SetterAwareType):
    __slots__ = []

    @setter
    def attr(self, value):
        return value * 2

    @setter
    def with_doc(self, value):
        """The doc string."""
        return value


class Example(ExampleWithSlots):
    pass


class TestSetter(unittest.TestCase):

    def setUp(self):
        self.item = Example()

    def test_setting(self):
        self.item.attr = 1
        assert_equal(self.item.attr, 2)

    def test_notset(self):
        assert_raises(AttributeError, getattr, self.item, 'attr')

    def test_set_other_attr(self):
        self.item.other_attr = 1
        assert_equal(self.item.other_attr, 1)

    def test_copy_doc(self):
        assert_equal(type(self.item).attr.__doc__, None)
        assert_equal(type(self.item).with_doc.__doc__, "The doc string.")


class TestSetterWithSlotsAndSetterAwareType(TestSetter):

    def setUp(self):
        self.item = ExampleWithSlots()

    def test_set_other_attr(self):
        assert_raises(AttributeError, setattr, self.item, 'other_attr', 1)

    def test_slots_as_tuple(self):
        class XY(metaclass=SetterAwareType):
            __slots__ = ('x',)

            def __init__(self, x, y):
                self.x = x
                self.y = y

            @setter
            def y(self, y):
                return y.upper()

        xy = XY('x', 'y')
        assert_equal((xy.x, xy.y), ('x', 'Y'))
        assert_raises(AttributeError, setattr, xy, 'z', 'z')


if __name__ == '__main__':
    unittest.main()



import re

from robot.api import TestSuite


EXTENSION = 'CUSTOM'
extension = 'ignored'


def parse(source):
    suite = TestSuite(source=source, metadata={'Parser': 'Custom'})
    for line in source.read_text().splitlines():
        if not line or line[0] in ('*', '#'):
            continue
        if line[0] != ' ':
            suite.tests.create(name=line)
        else:
            name, *args = re.split(r'\s{2,}', line.strip())
            suite.tests[-1].body.create_keyword(name, args)
    return suite



import unittest
import os

from robot.utils.asserts import assert_equal, assert_not_none, assert_none, assert_true
from robot.utils import get_env_var, set_env_var, del_env_var, get_env_vars


TEST_VAR = 'TeST_EnV_vAR'
TEST_VAL = 'original value'
NON_ASCII_VAR = u'\xe4iti'
NON_ASCII_VAL = u'is\xe4'


class TestRobotEnv(unittest.TestCase):

    def setUp(self):
        os.environ[TEST_VAR] = TEST_VAL

    def tearDown(self):
        if TEST_VAR in os.environ:
            del os.environ[TEST_VAR]

    def test_get_env_var(self):
        assert_not_none(get_env_var('PATH'))
        assert_equal(get_env_var(TEST_VAR), TEST_VAL)
        assert_none(get_env_var('NoNeXiStInG'))
        assert_equal(get_env_var('NoNeXiStInG', 'default'), 'default')

    def test_set_env_var(self):
        set_env_var(TEST_VAR, 'new value')
        assert_equal(os.getenv(TEST_VAR), 'new value')

    def test_del_env_var(self):
        old = del_env_var(TEST_VAR)
        assert_none(os.getenv(TEST_VAR))
        assert_equal(old, TEST_VAL)
        assert_none(del_env_var(TEST_VAR))

    def test_get_set_del_non_ascii_vars(self):
        set_env_var(NON_ASCII_VAR, NON_ASCII_VAL)
        assert_equal(get_env_var(NON_ASCII_VAR), NON_ASCII_VAL)
        assert_equal(del_env_var(NON_ASCII_VAR), NON_ASCII_VAL)
        assert_none(get_env_var(NON_ASCII_VAR))

    def test_get_env_vars(self):
        set_env_var(NON_ASCII_VAR, NON_ASCII_VAL)
        vars = get_env_vars()
        assert_true('PATH' in vars)
        assert_equal(vars[self._upper_on_windows(TEST_VAR)], TEST_VAL)
        assert_equal(vars[self._upper_on_windows(NON_ASCII_VAR)], NON_ASCII_VAL)
        for k, v in vars.items():
            assert_true(isinstance(k, str) and isinstance(v, str))

    def _upper_on_windows(self, name):
        return name if os.sep == '/' else name.upper()


if __name__ == '__main__':
    unittest.main()



no_args_vars = {
    'dyn_no_args_get_var': 'Dyn var got with no args from get_variables',
    'dyn_no_args_get_var_2': 2,
    'LIST__dyn_no_args_get_var_list': ['one', 2]
}
one_arg_vars = {
    'dyn_one_arg_get_var': 'Dyn var got with one arg from get_variables',
    'dyn_one_arg_get_var_False': False,
    'LIST__dyn_one_arg_get_var_list': ['one', False, no_args_vars]
}


def get_variables(*args):
    if len(args) == 0:
        return no_args_vars
    if len(args) == 1:
        return one_arg_vars
    if len(args) == 2:
        return None   # this is invalid
    raise Exception('Invalid arguments for get_variables')



class DynamicKwOnlyArgs:
    keywords = {
        'Args Should Have Been': ['*args', '**kwargs'],
        'Kw Only Arg': ['*', 'kwo'],
        'Many Kw Only Args': ['*', 'first', 'second', 'third'],
        'Kw Only Arg With Default': ['*', 'kwo=default', 'another=another'],
        'Mandatory After Defaults': ['*', 'default1=xxx', 'mandatory', 'default2=zzz'],
        'Kw Only Arg With Varargs': ['*varargs', 'kwo'],
        'All Arg Types': ['pos_req', 'pos_def=pd', '*varargs',
                          'kwo_req', 'kwo_def=kd', '**kwargs']
    }

    def __init__(self):
        self.args = self.kwargs = None

    def get_keyword_names(self):
        return list(self.keywords)

    def get_keyword_arguments(self, name):
        return self.keywords[name]

    def run_keyword(self, name, args, kwargs):
        if name != 'Args Should Have Been':
            self.args = args
            self.kwargs = kwargs
        elif self.args != args:
            raise AssertionError("Expected arguments %s, got %s."
                                 % (args, self.args))
        elif self.kwargs != kwargs:
            raise AssertionError("Expected kwargs %s, got %s."
                                 % (kwargs, self.kwargs))



from robot.api import Failure, Error


def failure(msg='I failed my duties', html=False):
    raise Failure(msg, html)


def error(msg='I errored my duties', html=False):
    raise Error(msg, html=html)



import unittest
import sys
import time
import os

from robot.errors import TimeoutError
from robot.running.timeouts import TestTimeout, KeywordTimeout
from robot.utils.asserts import (assert_equal, assert_false, assert_true,
                                 assert_raises, assert_raises_with_msg)

# thread_resources is here
sys.path.append(os.path.join(os.path.dirname(__file__),'..','utils'))
from thread_resources import passing, failing, sleeping, returning, MyException


class VariableMock:

    def replace_string(self, string):
        return string


class TestInit(unittest.TestCase):

    def test_no_params(self):
        self._verify_tout(TestTimeout())

    def test_timeout_string(self):
        for tout_str, exp_str, exp_secs in [ ('1s', '1 second', 1),
                                             ('10 sec', '10 seconds', 10),
                                             ('2h 1minute', '2 hours 1 minute', 7260),
                                             ('42', '42 seconds', 42) ]:
            self._verify_tout(TestTimeout(tout_str), exp_str, exp_secs)

    def test_invalid_timeout_string(self):
        for inv in ['invalid', '1s 1']:
            err = "Setting test timeout failed: Invalid time string '%s'."
            self._verify_tout(TestTimeout(inv), str=inv, secs=0.000001, err=err % inv)

    def _verify_tout(self, tout, str='', secs=-1, err=None):
        tout.replace_variables(VariableMock())
        assert_equal(tout.string, str)
        assert_equal(tout.secs, secs)
        assert_equal(tout.error, err)


class TestTimer(unittest.TestCase):

    def test_time_left(self):
        tout = TestTimeout('1s', variables=VariableMock())
        tout.start()
        assert_true(tout.time_left() > 0.9)
        time.sleep(0.2)
        assert_true(tout.time_left() < 0.9)

    def test_timed_out_with_no_timeout(self):
        tout = TestTimeout(variables=VariableMock())
        tout.start()
        time.sleep(0.01)
        assert_false(tout.timed_out())

    def test_timed_out_with_non_exceeded_timeout(self):
        tout = TestTimeout('10s', variables=VariableMock())
        tout.start()
        time.sleep(0.01)
        assert_false(tout.timed_out())

    def test_timed_out_with_exceeded_timeout(self):
        tout = TestTimeout('1ms', variables=VariableMock())
        tout.start()
        time.sleep(0.02)
        assert_true(tout.timed_out())


class TestComparisons(unittest.TestCase):

    def test_compare_when_none_timeouted(self):
        touts = self._create_timeouts([''] * 10)
        assert_equal(min(touts).string, '')
        assert_equal(max(touts).string, '')

    def test_compare_when_all_timeouted(self):
        touts = self._create_timeouts(['1min','42seconds','43','1h1min','99'])
        assert_equal(min(touts).string, '42 seconds')
        assert_equal(max(touts).string, '1 hour 1 minute')

    def test_compare_with_timeouted_and_non_timeouted(self):
        touts = self._create_timeouts(['','1min','42sec','','43','1h1m','99',''])
        assert_equal(min(touts).string, '42 seconds')
        assert_equal(max(touts).string, '')

    def test_that_compare_uses_starttime(self):
        touts = self._create_timeouts(['1min','42seconds','43','1h1min','99'])
        touts[2].starttime -= 2
        assert_equal(min(touts).string, '43 seconds')
        assert_equal(max(touts).string, '1 hour 1 minute')

    def _create_timeouts(self, tout_strs):
        touts = []
        for tout_str in tout_strs:
            touts.append(TestTimeout(tout_str, variables=VariableMock()))
            touts[-1].start()
        return touts


class TestRun(unittest.TestCase):

    def setUp(self):
        self.tout = TestTimeout('1s', variables=VariableMock())
        self.tout.start()

    def test_passing(self):
        assert_equal(self.tout.run(passing), None)

    def test_returning(self):
        for arg in [10, 'hello', ['l','i','s','t'], unittest]:
            ret = self.tout.run(returning, args=(arg,))
            assert_equal(ret, arg)

    def test_failing(self):
        assert_raises_with_msg(MyException, 'hello world',
                               self.tout.run, failing, ('hello world',))

    def test_sleeping(self):
        assert_equal(self.tout.run(sleeping, args=(0.01,)), 0.01)

    def test_method_executed_normally_if_no_timeout(self):
        os.environ['ROBOT_THREAD_TESTING'] = 'initial value'
        self.tout.run(sleeping, (0.05,))
        assert_equal(os.environ['ROBOT_THREAD_TESTING'], '0.05')

    def test_method_stopped_if_timeout(self):
        os.environ['ROBOT_THREAD_TESTING'] = 'initial value'
        self.tout.secs = 0.001
        # PyThreadState_SetAsyncExc thrown exceptions are not guaranteed
        # to occur in a specific timeframe ,, thus the actual Timeout exception
        # maybe thrown too late in Windows.
        # This is why we need to have an action that really will take some time (sleep 5 secs)
        # to (almost) ensure that the 'ROBOT_THREAD_TESTING' setting is not executed before
        # timeout exception occurs
        assert_raises_with_msg(TimeoutError, 'Test timeout 1 second exceeded.',
                               self.tout.run, sleeping, (5,))
        assert_equal(os.environ['ROBOT_THREAD_TESTING'], 'initial value')

    def test_zero_and_negative_timeout(self):
        for tout in [0, 0.0, -0.01, -1, -1000]:
            self.tout.time_left = lambda: tout
            assert_raises(TimeoutError, self.tout.run, sleeping, (10,))


class TestMessage(unittest.TestCase):

    def test_non_active(self):
        assert_equal(TestTimeout().get_message(), 'Test timeout not active.')

    def test_active(self):
        tout = KeywordTimeout('42s', variables=VariableMock())
        tout.start()
        msg = tout.get_message()
        assert_true(msg.startswith('Keyword timeout 42 seconds active.'), msg)
        assert_true(msg.endswith('seconds left.'), msg)

    def test_failed_default(self):
        tout = TestTimeout('1s', variables=VariableMock())
        tout.starttime = time.time() - 2
        assert_equal(tout.get_message(), 'Test timeout 1 second exceeded.')


if __name__ == '__main__':
    unittest.main()



from io import StringIO
import unittest

from robot.reporting.jsexecutionresult import JsExecutionResult
from robot.reporting.jswriter import JsResultWriter
from robot.utils.asserts import assert_equal, assert_true


def get_lines(suite=(), strings=(), basemillis=100, start_block='',
              end_block='', split_threshold=9999, min_level='INFO'):
    output = StringIO()
    data = JsExecutionResult(suite, None, None, strings, basemillis, min_level=min_level)
    writer = JsResultWriter(output, start_block, end_block, split_threshold)
    writer.write(data, settings={})
    return output.getvalue().splitlines()


def assert_separators(lines, separator, end_separator=False):
    for index, line in enumerate(lines):
        if index % 2 == int(end_separator):
            assert_equal(line, separator)
        else:
            assert_true(line.startswith('window.'), line)


class TestDataModelWrite(unittest.TestCase):

    def test_writing_datamodel_elements(self):
        lines = get_lines(min_level='DEBUG')
        assert_true(lines[0].startswith('window.output = {}'), lines[0])
        assert_true(lines[1].startswith('window.output["'), lines[1])
        assert_true(lines[-1].startswith('window.settings ='), lines[-1])

    def test_writing_datamodel_with_separator(self):
        lines = get_lines(start_block='seppo\n')
        assert_true(len(lines) >= 2)
        assert_separators(lines, 'seppo')

    def test_splitting_output_strings(self):
        lines = get_lines(strings=['data' for _ in range(100)],
                                split_threshold=9, end_block='?\n')
        parts = [l for l in lines if l.startswith('window.output["strings')]
        assert_equal(len(parts), 13)
        assert_equal(parts[0], 'window.output["strings"] = [];')
        for line in parts[1:]:
            assert_true(line.startswith('window.output["strings"] = window.output["strings"].concat(['), line)
        assert_separators(lines, '?', end_separator=True)


class TestSuiteWriter(unittest.TestCase):

    def test_no_splitting(self):
        suite = (1, (2, 3), (4, (5,), (6, ())), 8)
        expected = ['window.output["suite"] = [1,[2,3],[4,[5],[6,[]]],8];']
        self._assert_splitting(suite, 100, expected)

    def test_simple_splitting_version_1(self):
        suite = ((1, 2, 3, 4), (5, 6, 7, 8), 9)
        expected = ['window.sPart0 = [1,2,3,4];',
                    'window.sPart1 = [5,6,7,8];',
                    'window.output["suite"] = [window.sPart0,window.sPart1,9];']
        self._assert_splitting(suite, 4, expected)

    def test_simple_splitting_version_2(self):
        suite = ((1, 2, 3, 4), (5, 6, 7, 8), 9, 10)
        expected = ['window.sPart0 = [1,2,3,4];',
                    'window.sPart1 = [5,6,7,8];',
                    'window.sPart2 = [window.sPart0,window.sPart1,9,10];',
                    'window.output["suite"] = window.sPart2;']
        self._assert_splitting(suite, 4, expected)

    def test_simple_splitting_version_3(self):
        suite = ((1, 2, 3, 4), (5, 6, 7, 8, 9, 10), 11)
        expected = ['window.sPart0 = [1,2,3,4];',
                    'window.sPart1 = [5,6,7,8,9,10];',
                    'window.output["suite"] = [window.sPart0,window.sPart1,11];']
        self._assert_splitting(suite, 4, expected)

    def test_tuple_itself_has_size_one(self):
        suite = ((1, (), (), 4), (((((),),),),))
        expected = ['window.sPart0 = [1,[],[],4];',
                    'window.sPart1 = [[[[[]]]]];',
                    'window.output["suite"] = [window.sPart0,window.sPart1];']
        self._assert_splitting(suite, 4, expected)

    def test_nested_splitting(self):
        suite = (1, (2, 3), (4, (5,), (6, 7)), 8)
        expected = ['window.sPart0 = [2,3];',
                    'window.sPart1 = [6,7];',
                    'window.sPart2 = [4,[5],window.sPart1];',
                    'window.sPart3 = [1,window.sPart0,window.sPart2,8];',
                    'window.output["suite"] = window.sPart3;']
        self._assert_splitting(suite, 2, expected)

    def _assert_splitting(self, suite, threshold, expected):
        lines = get_lines(suite, split_threshold=threshold, start_block='foo\n')
        parts = [l for l in lines if l.startswith(('window.sPart',
                                                   'window.output["suite"]'))]
        assert_equal(parts, expected)
        assert_separators(lines, 'foo')


if __name__ == '__main__':
    unittest.main()



import sys


def get_variables():
    variables = {'integer': 42,
                 'float': 3.14,
                 'byte_string': b'hyv\xe4',
                 'byte_string_str': 'hyv\\xe4',
                 'boolean': True,
                 'none': None,
                 'module': sys,
                 'module_str': str(sys),
                 'list': [1, b'\xe4', '\xe4'],
                 'dict': {b'\xe4': '\xe4'},
                 'list_str': u"[1, b'\\xe4', '\xe4']",
                 'dict_str': u"{b'\\xe4': '\xe4'}"}
    return variables




import sys


def read_all():
    fails = 0
    for line in sys.stdin:
        if 'FAIL' in line:
            fails += 1
    print("%d lines with 'FAIL' found!" % fails)


def read_some():
    for line in sys.stdin:
        if 'FAIL' in line:
            print("Line with 'FAIL' found!")
            sys.stdin.close()
            break


def read_none():
    sys.stdin.close()


globals()[sys.argv[1]]()



from robot.api.deco import keyword, library


class Listener:
    ROBOT_LISTENER_API_VERSION = 3


@library(scope='TEST SUITE', version='1.2.3', listener=Listener())
class LibraryDecoratorWithArgs:

    def not_keyword_v2(self):
        raise RuntimeError('Should not be executed!')

    @keyword(name="Decorated method is keyword v.2")
    def decorated_method_is_keyword(self):
        print('Decorated methods are keywords.')



from robot.api.deco import keyword


@keyword('${match} in ${both} libraries')
def match_in_both_libraries(match, both):
    assert match == 'Match'
    assert both == 'both'

@keyword('Follow search ${order} in libraries')
def follow_search_order_in_libraries(order):
    assert order == 'order'

@keyword('${match} libraries')
def match_libraries(match):
    assert False


@keyword('Unresolvable ${conflict} in library')
def unresolvable_conflict_in_library(conflict):
    assert False


@keyword('${possible} conflict in library')
def possible_conflict_in_library(possible):
    assert possible == 'No'



import unittest

from robot.utils.asserts import assert_raises, assert_equal
from robot.errors import DataError
from robot.output.loggerhelper import AbstractLogger, Message


class TestAbstractLogger(unittest.TestCase):

    def test_set_invalid_threshold(self):
        logger = AbstractLogger('trace')
        assert_raises(DataError, logger.set_level, 'INVALID THRESHOLD')


class TestMessage(unittest.TestCase):

    def test_string_message(self):
        assert_equal(Message('my message').message, 'my message')

    def test_callable_message(self):
        assert_equal(Message(lambda: 'my message').message, 'my message')


if __name__ == '__main__':
    unittest.main()



class KwargsLibrary:

    def one_named(self, named=None):
        return named

    def two_named(self, fst=None, snd=None):
        return '%s, %s' % (fst, snd)

    def four_named(self, a=None, b=None, c=None, d=None):
        return '%s, %s, %s, %s' % (a, b, c, d)

    def mandatory_and_named(self, a, b, c=None):
        return '%s, %s, %s' % (a, b, c)

    def mandatory_named_and_varargs(self, mandatory, d1=None, d2=None, *varargs):
        return '%s, %s, %s, %s' % (mandatory, d1, d2, '[%s]' % ', '.join(varargs))



import subprocess
import sys
import time


def countdown(path):
    for i in range(10, 0, -1):
        with open(path, 'w') as f:
            f.write('%d\n' % i)
        time.sleep(0.2)
    with open(path, 'w') as f:
        f.write('BLASTOFF')


if __name__ == '__main__':
    path = sys.argv[1]
    children = int(sys.argv[2]) if len(sys.argv) == 3 else 0

    if children:
        subprocess.Popen([sys.executable, __file__, path, str(children-1)]).wait()
    else:
        countdown(path)



def getVariables(*args):
    variables = {
      'dyn_multi_args_getVar' : 'Dyn var got with multiple args from getVariables',
      'dyn_multi_args_getVar_x' : ' '.join([str(a) for a in args])
    }
    return variables


class DynamicPositionalOnly:
    kws = {
        "one argument": ["one", "/"],
        "three arguments": ["a", "b", "c", "/"],
        "with normal": ["posonly", "/", "normal"],
        "default str": ["required", "optional=default", "/"],
        "default tuple": ["required", ("optional", "default"), "/"],
        "all args kw": [("one", "value"), "/", ("named", "other"), "*varargs", "**kwargs"],
        "arg with separator": ["/one"],
        "Too many markers": ["one", "/", "two", "/"],
        "After varargs": ["*varargs", "/", "arg"],
        "After named-only marker": ["*", "/", "arg"],
        "After kwargs": ["**kws", "/"],
    }

    def get_keyword_names(self):
        return [key for key in self.kws]

    def run_keyword(self, name, args, kwargs=None):
        if kwargs:
            return f"{name}-{args}-{kwargs}"
        return f"{name}-{args}"

    def get_keyword_arguments(self, name):
        return self.kws[name]



from robot.errors import PassExecution
from robot.libraries.BuiltIn import BuiltIn


def raise_pass_execution_exception(msg):
    raise PassExecution(msg)


def call_pass_execution_method(msg):
    BuiltIn().pass_execution(msg, 'lol')



from robot.api.deco import keyword


@keyword('Get ${Match} With Search Order')
def get_best_match_ever_with_search_order(Match):
    raise AssertionError('Should not be run due to a better match'
                         'in same library.')

@keyword('Get Best ${Match:\w+} With Search Order')
def get_best_match_with_search_order(Match):
    raise AssertionError('Should not be run due to a better match'
                         'in same library.')

@keyword('Get Best ${Match} With Search Order')
def get_best_match_with_search_order(Match):
    assert Match == "Match Ever"
    return "embedded2"



import unittest

from robot.running import TestSuite, TestCase
from robot.utils.asserts import assert_equal, assert_not_equal

class TestRandomizing(unittest.TestCase):
    names = [str(i) for i in range(100)]

    def setUp(self):
        self.suite = self._generate_suite()

    def _generate_suite(self):
        s = TestSuite()
        s.suites = self._generate_suites()
        s.tests  = self._generate_tests()
        return s

    def _generate_suites(self):
        return [TestSuite(name=n) for n in self.names]

    def _generate_tests(self):
        return [TestCase(name=n) for n in self.names]

    def _assert_randomized(self, items):
        assert_not_equal([i.name for i in items], self.names)

    def _assert_not_randomized(self, items):
        assert_equal([i.name for i in items], self.names)

    def test_randomize_nothing(self):
        self.suite.randomize(suites=False, tests=False)
        self._assert_not_randomized(self.suite.suites)
        self._assert_not_randomized(self.suite.tests)

    def test_randomize_only_suites(self):
        self.suite.randomize(suites=True, tests=False)
        self._assert_randomized(self.suite.suites)
        self._assert_not_randomized(self.suite.tests)

    def test_randomize_only_tests(self):
        self.suite.randomize(suites=False, tests=True)
        self._assert_not_randomized(self.suite.suites)
        self._assert_randomized(self.suite.tests)

    def test_randomize_both(self):
        self.suite.randomize(suites=True, tests=True)
        self._assert_randomized(self.suite.suites)
        self._assert_randomized(self.suite.tests)

    def test_randomize_recursively(self):
        self.suite.suites[0].suites = self._generate_suites()
        self.suite.suites[1].tests = self._generate_tests()
        self.suite.randomize(suites=True, tests=True)
        self._assert_randomized(self.suite.suites[0].suites)
        self._assert_randomized(self.suite.suites[1].tests)

    def test_randomizing_changes_ids(self):
        assert_equal([s.id for s in self.suite.suites],
                      ['s1-s%d' % i for i in range(1, 101)])
        assert_equal([t.id for t in self.suite.tests],
                      ['s1-t%d' % i for i in range(1, 101)])
        self.suite.randomize(suites=True, tests=True)
        assert_equal([s.id for s in self.suite.suites],
                      ['s1-s%d' % i for i in range(1, 101)])
        assert_equal([t.id for t in self.suite.tests],
                      ['s1-t%d' % i for i in range(1, 101)])

    def _gen_random_suite(self, seed):
        suite = self._generate_suite()
        suite.randomize(suites=True, tests=True, seed=seed)
        random_order_suites = [i.name for i in suite.suites]
        random_order_tests  = [i.name for i in suite.tests]
        return (random_order_suites, random_order_tests)

    def test_randomize_seed(self):
        """
        GIVEN a test suite
        WHEN it's randomized with a given seed
        THEN it's always sorted in the same order
        """
        (random_order_suites1, random_order_tests1) = self._gen_random_suite(1234)
        (random_order_suites2, random_order_tests2) = self._gen_random_suite(1234)
        assert_equal( random_order_suites1, random_order_suites2 )
        assert_equal( random_order_tests1, random_order_tests2 )

if __name__ == '__main__':
    unittest.main()



from collections import abc
from datetime import datetime, date, timedelta
from decimal import Decimal
from enum import Flag, Enum, IntFlag, IntEnum
from fractions import Fraction    # Needed by `eval()` in `_validate_type()`.
from numbers import Integral, Real
from os import PathLike
from pathlib import Path, PurePath
from typing import Union

from robot.api.deco import keyword


class MyEnum(Enum):
    FOO = 1
    bar = 'xxx'
    foo = 'yyy'
    normalize_me = True


class MyFlag(Flag):
    RED = 1
    BLUE = 2


class MyIntEnum(IntEnum):
    ON = 1
    OFF = 0


class MyIntFlag(IntFlag):
    R = 4
    W = 2
    X = 1


class Unknown:
    pass


@keyword(types={'argument': int})
def integer(argument, expected=None):
    _validate_type(argument, expected)


@keyword(types={'argument': Integral})
def integral(argument, expected=None):
    _validate_type(argument, expected)


@keyword(types={'argument': float})
def float_(argument, expected=None):
    _validate_type(argument, expected)


@keyword(types={'argument': Real})
def real(argument, expected=None):
    _validate_type(argument, expected)


@keyword(types={'argument': Decimal})
def decimal(argument, expected=None):
    _validate_type(argument, expected)


@keyword(types={'argument': bool})
def boolean(argument, expected=None):
    _validate_type(argument, expected)


@keyword(types={'argument': str})
def string(argument, expected=None):
    _validate_type(argument, expected)


@keyword(types={'argument': bytes})
def bytes_(argument, expected=None):
    _validate_type(argument, expected)


@keyword(types={'argument': getattr(abc, 'ByteString', None)})
def bytestring(argument, expected=None):
    _validate_type(argument, expected)


@keyword(types={'argument': bytearray})
def bytearray_(argument, expected=None):
    _validate_type(argument, expected)


@keyword(types={'argument': datetime})
def datetime_(argument, expected=None):
    _validate_type(argument, expected)


@keyword(types={'argument': date})
def date_(argument, expected=None):
    _validate_type(argument, expected)


@keyword(types={'argument': timedelta})
def timedelta_(argument, expected=None):
    _validate_type(argument, expected)


@keyword(types={'argument': Path})
def path(argument, expected=None):
    _validate_type(argument, expected)


@keyword(types={'argument': PurePath})
def pure_path(argument, expected=None):
    _validate_type(argument, expected)


@keyword(types={'argument': PathLike})
def path_like(argument, expected=None):
    _validate_type(argument, expected)


@keyword(types={'argument': MyEnum})
def enum(argument, expected=None):
    _validate_type(argument, expected)


@keyword(types={'argument': MyFlag})
def flag(argument, expected=None):
    _validate_type(argument, expected)


@keyword(types=[MyIntEnum])
def int_enum(argument, expected=None):
    _validate_type(argument, expected)


@keyword(types=[MyIntFlag])
def int_flag(argument, expected=None):
    _validate_type(argument, expected)


@keyword(types={'argument': type(None)})
def nonetype(argument, expected=None):
    _validate_type(argument, expected)


@keyword(types={'argument': None})
def none(argument, expected=None):
    _validate_type(argument, expected)


@keyword(types={'argument': list})
def list_(argument, expected=None):
    _validate_type(argument, expected)


@keyword(types={'argument': abc.Sequence})
def sequence(argument, expected=None):
    _validate_type(argument, expected)


@keyword(types={'argument': abc.MutableSequence})
def mutable_sequence(argument, expected=None):
    _validate_type(argument, expected)


@keyword(types={'argument': tuple})
def tuple_(argument, expected=None):
    _validate_type(argument, expected)


@keyword(types={'argument': dict})
def dictionary(argument, expected=None):
    _validate_type(argument, expected)


@keyword(types={'argument': abc.Mapping})
def mapping(argument, expected=None):
    _validate_type(argument, expected)


@keyword(types={'argument': abc.MutableMapping})
def mutable_mapping(argument, expected=None):
    _validate_type(argument, expected)


@keyword(types={'argument': set})
def set_(argument, expected=None):
    _validate_type(argument, expected)


@keyword(types={'argument': abc.Set})
def set_abc(argument, expected=None):
    _validate_type(argument, expected)


@keyword(types={'argument': abc.MutableSet})
def mutable_set(argument, expected=None):
    _validate_type(argument, expected)


@keyword(types={'argument': frozenset})
def frozenset_(argument, expected=None):
    _validate_type(argument, expected)


@keyword(types={'argument': Unknown})
def unknown(argument, expected=None):
    _validate_type(argument, expected)


@keyword(types={'argument': 'this is string, not type'})
def non_type(argument, expected=None):
    _validate_type(argument, expected)


@keyword(types={'argument': int})
def varargs(*argument, **expected):
    expected = expected.pop('expected', None)
    _validate_type(argument, expected)


@keyword(types={'argument': int})
def kwargs(expected=None, **argument):
    _validate_type(argument, expected)


@keyword(types={'argument': float})
def kwonly(*, argument, expected=None):
    _validate_type(argument, expected)


@keyword(types='invalid')
def invalid_type_spec():
    raise RuntimeError('Should not be executed')


@keyword(types={'no_match': int, 'xxx': 42})
def non_matching_name(argument):
    raise RuntimeError('Should not be executed')


@keyword(types={'argument': int, 'return': float})
def return_type(argument, expected=None):
    _validate_type(argument, expected)


@keyword(types=[list])
def type_and_default_1(argument=None, expected=None):
    _validate_type(argument, expected)


@keyword(types=[int])
def type_and_default_2(argument=True, expected=None):
    _validate_type(argument, expected)


@keyword(types=[timedelta])
def type_and_default_3(argument=0, expected=None):
    _validate_type(argument, expected)


@keyword(types={'argument': Union[int, None, float]})
def multiple_types_using_union(argument, expected=None):
    _validate_type(argument, expected)


@keyword(types={'argument': (int, None, float)})
def multiple_types_using_tuple(argument, expected=None):
    _validate_type(argument, expected)


def _validate_type(argument, expected):
    if isinstance(expected, str):
        expected = eval(expected)
    if argument != expected or type(argument) != type(expected):
        raise AssertionError('%r (%s) != %r (%s)'
                             % (argument, type(argument).__name__,
                                expected, type(expected).__name__))



import os

THIS_PATH = os.path.dirname(__file__)
GOLDEN_OUTPUT = os.path.join(THIS_PATH, 'golden_suite', 'output.xml')
GOLDEN_OUTPUT2 = os.path.join(THIS_PATH, 'golden_suite', 'output2.xml')
GOLDEN_JS = os.path.join(THIS_PATH, 'golden_suite', 'expected.js')




import unittest
import warnings
from datetime import datetime

from robot.model import Tags
from robot.result import (Break, Continue, Error, For, If, IfBranch, Keyword, Message,
                          Return, TestCase, TestSuite, Try, While)
from robot.utils.asserts import (assert_equal, assert_false, assert_raises,
                                 assert_raises_with_msg, assert_true)


class TestSuiteStats(unittest.TestCase):

    def test_stats(self):
        suite = self._create_suite_with_tests()
        assert_equal(suite.statistics.passed, 3)
        assert_equal(suite.statistics.failed, 2)
        assert_equal(suite.statistics.skipped, 1)

    def test_nested_suite_stats(self):
        suite = self._create_nested_suite_with_tests()
        assert_equal(suite.statistics.passed, 6)
        assert_equal(suite.statistics.failed, 4)
        assert_equal(suite.statistics.skipped, 2)

    def test_test_count(self):
        suite = self._create_nested_suite_with_tests()
        assert_equal(suite.test_count, 12)
        assert_equal(suite.suites[0].test_count, 6)
        suite.suites.append(self._create_suite_with_tests())
        assert_equal(suite.test_count, 18)
        assert_equal(suite.suites[-1].test_count, 6)
        suite.suites[-1].tests.create()
        assert_equal(suite.test_count, 19)
        assert_equal(suite.suites[-1].test_count, 7)

    def _create_nested_suite_with_tests(self):
        suite = TestSuite()
        suite.suites = [self._create_suite_with_tests(),
                        self._create_suite_with_tests()]
        return suite

    def _create_suite_with_tests(self):
        suite = TestSuite()
        suite.tests = [TestCase(status='PASS'),
                       TestCase(status='PASS'),
                       TestCase(status='PASS'),
                       TestCase(status='FAIL'),
                       TestCase(status='FAIL'),
                       TestCase(status='SKIP')]
        return suite


class TestSuiteStatus(unittest.TestCase):

    def test_suite_status_is_skip_if_there_are_no_tests(self):
        assert_equal(TestSuite().status, 'SKIP')

    def test_suite_status_is_fail_if_failed_test(self):
        suite = TestSuite()
        suite.tests.create(status='PASS')
        assert_equal(suite.status, 'PASS')
        suite.tests.create(status='FAIL')
        assert_equal(suite.status, 'FAIL')
        suite.tests.create(status='PASS')
        assert_equal(suite.status, 'FAIL')

    def test_suite_status_is_pass_if_only_passed_tests(self):
        suite = TestSuite()
        for i in range(10):
            suite.tests.create(status='PASS')
        assert_equal(suite.status, 'PASS')

    def test_suite_status_is_pass_if_passed_and_skipped(self):
        suite = TestSuite()
        for i in range(5):
            suite.tests.create(status='PASS')
            suite.tests.create(status='SKIP')
        assert_equal(suite.status, 'PASS')

    def test_suite_status_is_skip_if_only_skipped_tests(self):
        suite = TestSuite()
        for i in range(10):
            suite.tests.create(status='SKIP')
        assert_equal(suite.status, 'SKIP')
        assert_true(suite.skipped)

    def test_suite_status_is_fail_if_failed_subsuite(self):
        suite = TestSuite()
        suite.suites.create().tests.create(status='FAIL')
        assert_equal(suite.status, 'FAIL')
        suite.tests.create(status='PASS')
        assert_equal(suite.status, 'FAIL')

    def test_status_propertys(self):
        suite = TestSuite()
        assert_false(suite.passed)
        assert_false(suite.failed)
        assert_true(suite.skipped)
        assert_false(suite.not_run)
        suite.tests.create(status='SKIP')
        assert_false(suite.passed)
        assert_false(suite.failed)
        assert_true(suite.skipped)
        assert_false(suite.not_run)
        suite.tests.create(status='PASS')
        assert_true(suite.passed)
        assert_false(suite.failed)
        assert_false(suite.skipped)
        assert_false(suite.not_run)
        suite.tests.create(status='FAIL')
        assert_false(suite.passed)
        assert_true(suite.failed)
        assert_false(suite.skipped)
        assert_false(suite.not_run)

    def test_suite_status_cannot_be_set_directly(self):
        suite = TestSuite()
        for attr in 'status', 'passed', 'failed', 'skipped', 'not_run':
            assert_true(hasattr(suite, attr))
            assert_raises(AttributeError, setattr, suite, attr, True)


class TestTimes(unittest.TestCase):

    def test_suite_elapsed_time_when_start_and_end_given(self):
        suite = TestSuite()
        suite.starttime = '20010101 10:00:00.000'
        suite.endtime = '20010101 10:00:01.234'
        self.assert_elapsed(suite, 1234)

    def assert_elapsed(self, obj, expected):
        assert_equal(obj.elapsedtime, expected)
        assert_equal(obj.elapsed_time.total_seconds() * 1000, expected)

    def test_suite_elapsed_time_is_zero_by_default(self):
        self.assert_elapsed(TestSuite(), 0)

    def test_suite_elapsed_time_is_got_from_childen_if_suite_does_not_have_times(self):
        suite = TestSuite()
        suite.tests.create(starttime='19991212 12:00:00.010',
                           endtime='19991212 12:00:00.011')
        self.assert_elapsed(suite, 1)
        assert_equal(suite.elapsedtime, 1)
        suite.starttime = '19991212 12:00:00.010'
        suite.endtime = '19991212 12:00:01.010'
        self.assert_elapsed(suite, 1000)

    def test_forward_compatibility(self):
        for cls in (TestSuite, TestCase, Keyword, If, IfBranch, Try, For, While,
                    Break, Continue, Return, Error):
            obj = cls(starttime='20230512 16:40:00.001', endtime='20230512 16:40:01.001')
            assert_equal(obj.starttime, '20230512 16:40:00.001')
            assert_equal(obj.endtime, '20230512 16:40:01.001')
            assert_equal(obj.start_time, datetime(2023, 5, 12, 16, 40, 0, 1000))
            assert_equal(obj.end_time, datetime(2023, 5, 12, 16, 40, 1, 1000))
            self.assert_elapsed(obj, 1000)


class TestSlots(unittest.TestCase):

    def test_testsuite(self):
        self._verify(TestSuite())

    def test_testcase(self):
        self._verify(TestCase())

    def test_keyword(self):
        self._verify(Keyword())

    def test_if(self):
        self._verify(If())
        self._verify(If().body.create_branch())

    def test_for(self):
        self._verify(For())
        self._verify(For().body.create_iteration())

    def test_try(self):
        self._verify(Try())
        self._verify(Try().body.create_branch())

    def test_while(self):
        self._verify(While())
        self._verify(While().body.create_iteration())

    def test_break_continue_return(self):
        for cls in Break, Continue, Return:
            self._verify(cls())

    def test_error(self):
        self._verify(Error())

    def test_message(self):
        self._verify(Message())

    def _verify(self, item):
        assert_raises(AttributeError, setattr, item, 'attr', 'value')


class TestModel(unittest.TestCase):

    def test_keyword_name(self):
        kw = Keyword('keyword')
        assert_equal(kw.name, 'keyword')
        kw = Keyword('keyword', 'lib')
        assert_equal(kw.name, 'lib.keyword')
        kw.kwname = 'Kekkonen'
        kw.libname = 'Urho'
        assert_equal(kw.name, 'Urho.Kekkonen')

    def test_keyword_name_cannot_be_set_directly(self):
        assert_raises(AttributeError, setattr, Keyword(), 'name', 'value')

    def test_status_propertys_with_test(self):
        self._verify_status_propertys(TestCase())

    def test_status_propertys_with_keyword(self):
        self._verify_status_propertys(Keyword())

    def test_status_propertys_with_control_structures(self):
        for obj in (Break(), Continue(), Return(), Error(),
                    For(), For().body.create_iteration(),
                    If(), If().body.create_branch(),
                    Try(), Try().body.create_branch(),
                    While(), While().body.create_iteration()):
            self._verify_status_propertys(obj)

    def test_keyword_passed_after_dry_run(self):
        self._verify_status_propertys(Keyword(status=Keyword.NOT_RUN),
                                      initial_status=Keyword.NOT_RUN)

    def _verify_status_propertys(self, item, initial_status='FAIL'):
        item.starttime = '20210121 17:04:00.000'
        item.endtime = '20210121 17:04:01.002'
        assert_equal(item.elapsedtime, 1002)
        assert_equal(item.passed, initial_status == item.PASS)
        assert_equal(item.failed, initial_status == item.FAIL)
        assert_equal(item.skipped, initial_status == item.SKIP)
        assert_equal(item.not_run, initial_status == item.NOT_RUN)
        assert_equal(item.status, initial_status)
        item.passed = True
        assert_equal(item.passed, True)
        assert_equal(item.failed, False)
        assert_equal(item.skipped, False)
        assert_equal(item.not_run, False)
        assert_equal(item.status, 'PASS')
        item.passed = False
        assert_equal(item.passed, False)
        assert_equal(item.failed, True)
        assert_equal(item.skipped, False)
        assert_equal(item.not_run, False)
        assert_equal(item.status, 'FAIL')
        item.failed = True
        assert_equal(item.passed, False)
        assert_equal(item.failed, True)
        assert_equal(item.skipped, False)
        assert_equal(item.not_run, False)
        assert_equal(item.status, 'FAIL')
        item.failed = False
        assert_equal(item.passed, True)
        assert_equal(item.failed, False)
        assert_equal(item.skipped, False)
        assert_equal(item.not_run, False)
        assert_equal(item.status, 'PASS')
        item.skipped = True
        assert_equal(item.passed, False)
        assert_equal(item.failed, False)
        assert_equal(item.skipped, True)
        assert_equal(item.not_run, False)
        assert_equal(item.status, 'SKIP')
        assert_raises(ValueError, setattr, item, 'skipped', False)
        if isinstance(item, TestCase):
            assert_raises(AttributeError, setattr, item, 'not_run', True)
            assert_raises(AttributeError, setattr, item, 'not_run', False)
        else:
            item.not_run = True
            assert_equal(item.passed, False)
            assert_equal(item.failed, False)
            assert_equal(item.skipped, False)
            assert_equal(item.not_run, True)
            assert_equal(item.status, 'NOT RUN')
            assert_raises(ValueError, setattr, item, 'not_run', False)

    def test_keyword_teardown(self):
        kw = Keyword()
        assert_true(not kw.has_teardown)
        assert_true(not kw.teardown)
        assert_equal(kw.teardown.name, None)
        assert_equal(kw.teardown.type, 'TEARDOWN')
        assert_true(not kw.has_teardown)
        assert_true(not kw.teardown)
        kw.teardown = Keyword()
        assert_true(kw.has_teardown)
        assert_true(kw.teardown)
        assert_equal(kw.teardown.name, '')
        assert_equal(kw.teardown.type, 'TEARDOWN')
        kw.teardown = None
        assert_true(not kw.has_teardown)
        assert_true(not kw.teardown)
        assert_equal(kw.teardown.name, None)
        assert_equal(kw.teardown.type, 'TEARDOWN')

    def test_keywords_deprecation(self):
        kw = Keyword()
        kw.body = [Keyword(), Message(), Keyword(), Keyword(), Message()]
        kw.teardown.config(kwname='T')
        with warnings.catch_warnings(record=True) as w:
            kws = kw.keywords
            assert_equal(list(kws), [kw.body[0], kw.body[2], kw.body[3], kw.teardown])
            assert_true('deprecated' in str(w[0].message))
        assert_raises(AttributeError, kws.append, Keyword())
        assert_raises(AttributeError, setattr, kw, 'keywords', [])

    def test_for_parents(self):
        test = TestCase()
        for_ = test.body.create_for()
        assert_equal(for_.parent, test)
        iter1 = for_.body.create_iteration()
        assert_equal(iter1.parent, for_)
        kw = iter1.body.create_keyword()
        assert_equal(kw.parent, iter1)
        iter2 = for_.body.create_iteration()
        assert_equal(iter2.parent, for_)
        kw = iter2.body.create_keyword()
        assert_equal(kw.parent, iter2)

    def test_if_parents(self):
        test = TestCase()
        if_ = test.body.create_if()
        assert_equal(if_.parent, test)
        branch = if_.body.create_branch(if_.IF, '$x > 0')
        assert_equal(branch.parent, if_)
        kw = branch.body.create_keyword()
        assert_equal(kw.parent, branch)
        branch = if_.body.create_branch(if_.ELSE_IF, '$x < 0')
        assert_equal(branch.parent, if_)
        kw = branch.body.create_keyword()
        assert_equal(kw.parent, branch)
        branch = if_.body.create_branch(if_.ELSE)
        assert_equal(branch.parent, if_)
        kw = branch.body.create_keyword()
        assert_equal(kw.parent, branch)

    def test_while_name(self):
        assert_equal(While().name, '')
        assert_equal(While('$x > 0').name, '$x > 0')
        assert_equal(While('True', '1 minute').name, 'True | limit=1 minute')
        assert_equal(While(limit='1 minute').name, 'limit=1 minute')
        assert_equal(While('True', '1 s', on_limit_message='Error message').name,
                     'True | limit=1 s | on_limit_message=Error message')
        assert_equal(While(on_limit='pass').name,
                     'on_limit=pass')
        assert_equal(While(on_limit_message='Error message').name,
                     'on_limit_message=Error message')


class TestBody(unittest.TestCase):

    def test_only_keywords(self):
        kw = Keyword()
        for i in range(10):
            kw.body.create_keyword(str(i))
        assert_equal([k.name for k in kw.body], [str(i) for i in range(10)])

    def test_only_messages(self):
        kw = Keyword()
        for i in range(10):
            kw.body.create_message(str(i))
        assert_equal([m.message for m in kw.body], [str(i) for i in range(10)])

    def test_order(self):
        kw = Keyword()
        m1 = kw.body.create_message('m1')
        k1 = kw.body.create_keyword('k1')
        k2 = kw.body.create_keyword('k2')
        m2 = kw.body.create_message('m2')
        k3 = kw.body.create_keyword('k3')
        assert_equal(list(kw.body), [m1, k1, k2, m2, k3])

    def test_order_after_modifications(self):
        kw = Keyword('parent')
        kw.body.create_keyword('k1')
        kw.body.create_message('m1')
        k2 = kw.body.create_keyword('k2')
        m2 = kw.body.create_message('m2')
        k1 = kw.body[0] = Keyword('k1-new')
        m1 = kw.body[1] = Message('m1-new')
        m3 = Message('m3')
        kw.body.append(m3)
        k3 = Keyword('k3')
        kw.body.extend([k3])
        assert_equal(list(kw.body), [k1, m1, k2, m2, m3, k3])
        kw.body = [k3, m2, k1]
        assert_equal(list(kw.body), [k3, m2, k1])

    def test_id(self):
        kw = TestSuite().tests.create().body.create_keyword()
        kw.body = [Keyword(), Message(), Keyword()]
        kw.body[-1].body = [Message(), Keyword(), Message()]
        assert_equal(kw.body[0].id, 's1-t1-k1-k1')
        assert_equal(kw.body[1].id, 's1-t1-k1-m1')
        assert_equal(kw.body[2].id, 's1-t1-k1-k2')
        assert_equal(kw.body[2].body[0].id, 's1-t1-k1-k2-m1')
        assert_equal(kw.body[2].body[1].id, 's1-t1-k1-k2-k1')
        assert_equal(kw.body[2].body[2].id, 's1-t1-k1-k2-m2')


class TestIterations(unittest.TestCase):

    def test_create_supported(self):
        for parent in For(), While():
            iterations = parent.body
            for creator in (iterations.create_iteration,
                            iterations.create_message,
                            iterations.create_keyword):
                item = creator()
                assert_equal(item.parent, parent)

    def test_create_not_supported(self):
        msg = "'robot.result.Iterations' object does not support '{}'."
        for parent in For(), While():
            iterations = parent.body
            for creator in (iterations.create_for,
                            iterations.create_if,
                            iterations.create_try,
                            iterations.create_return):
                assert_raises_with_msg(TypeError, msg.format(creator.__name__), creator)


class TestBranches(unittest.TestCase):

    def test_create_supported(self):
        for parent in If(), Try():
            branches = parent.body
            for creator in (branches.create_branch,
                            branches.create_message,
                            branches.create_keyword):
                item = creator()
                assert_equal(item.parent, parent)

    def test_create_not_supported(self):
        msg = "'robot.result.Branches' object does not support '{}'."
        for parent in If(), Try():
            branches = parent.body
            for creator in (branches.create_for,
                            branches.create_if,
                            branches.create_try,
                            branches.create_return):
                assert_raises_with_msg(TypeError, msg.format(creator.__name__), creator)


class TestDeprecatedKeywordSpecificAttributes(unittest.TestCase):

    def test_deprecated_keyword_specific_properties(self):
        for_ = For(['${x}', '${y}'], 'IN', ['a', 'b', 'c', 'd'])
        for name, expected in [('name', '${x} | ${y} IN [ a | b | c | d ]'),
                               ('args', ()),
                               ('assign', ()),
                               ('tags', Tags()),
                               ('timeout', None)]:
            assert_equal(getattr(for_, name), expected)

    def test_if(self):
        for name, expected in [('name', ''),
                               ('args', ()),
                               ('assign', ()),
                               ('tags', Tags()),
                               ('timeout', None)]:
            assert_equal(getattr(If(), name), expected)

    def test_if_branch(self):
        branch = IfBranch(IfBranch.IF, '$x > 0')
        for name, expected in [('name', '$x > 0'),
                               ('args', ()),
                               ('assign', ()),
                               ('tags', Tags()),
                               ('timeout', None)]:
            assert_equal(getattr(branch, name), expected)


if __name__ == '__main__':
    unittest.main()



import unittest

from robot.api.deco import keyword, library
from robot.utils.asserts import assert_equal, assert_false


class TestKeywordName(unittest.TestCase):

    def test_give_name_to_function(self):
        @keyword('Given name')
        def func():
            pass
        assert_equal(func.robot_name, 'Given name')

    def test_give_name_to_method(self):
        class Class:
            @keyword('Given name')
            def method(self):
                pass
        assert_equal(Class.method.robot_name, 'Given name')

    def test_no_name(self):
        @keyword()
        def func():
            pass
        assert_equal(func.robot_name, None)

    def test_no_name_nor_parens(self):
        @keyword
        def func():
            pass
        assert_equal(func.robot_name, None)


class TestLibrary(unittest.TestCase):

    def test_auto_keywords_is_disabled_by_default(self):
        @library
        class lib1:
            pass
        @library()
        class lib2:
            pass
        self._validate_lib(lib1)
        self._validate_lib(lib2)

    def test_auto_keywords_can_be_enabled(self):
        @library(auto_keywords=False)
        class lib:
            pass
        self._validate_lib(lib, auto_keywords=False)

    def test_other_options(self):
        @library('GLOBAL', version='v', doc_format='HTML', listener='xx')
        class lib:
            pass
        self._validate_lib(lib, 'GLOBAL', 'v', 'HTML', 'xx')

    def test_override_class_level_attributes(self):
        @library(doc_format='HTML', listener='xx', scope='GLOBAL', version='v',
                 auto_keywords=True)
        class lib:
            ROBOT_LIBRARY_SCOPE = 'override'
            ROBOT_LIBRARY_VERSION = 'override'
            ROBOT_LIBRARY_DOC_FORMAT = 'override'
            ROBOT_LIBRARY_LISTENER = 'override'
            ROBOT_AUTO_KEYWORDS = 'override'
        self._validate_lib(lib, 'GLOBAL', 'v', 'HTML', 'xx', True)

    def _validate_lib(self, lib, scope=None, version=None, doc_format=None,
                      listener=None, auto_keywords=False):
        self._validate_attr(lib, 'ROBOT_LIBRARY_SCOPE', scope)
        self._validate_attr(lib, 'ROBOT_LIBRARY_VERSION', version)
        self._validate_attr(lib, 'ROBOT_LIBRARY_DOC_FORMAT', doc_format)
        self._validate_attr(lib, 'ROBOT_LIBRARY_LISTENER', listener)
        self._validate_attr(lib, 'ROBOT_AUTO_KEYWORDS', auto_keywords)

    def _validate_attr(self, lib, attr, value):
        if value is None:
            assert_false(hasattr(lib, attr))
        else:
            assert_equal(getattr(lib, attr), value)


if __name__ == '__main__':
    unittest.main()



def in_exception():
    raise Exception(b'hyv\xe4')

def in_return_value():
    return b'ty\xf6paikka'

def in_message():
    print(b'\xe4iti')

def in_multiline_message():
    print(b'\xe4iti\nis\xe4')



import unittest

from robot.utils.asserts import assert_equal

from robot.output.pyloggingconf import RobotHandler

import logging


class MessageMock:
    def __init__(self, timestamp, level, message):
        self.timestamp = timestamp
        self.level = level
        self.message = message


class MockLibraryLogger:
    def __init__(self):
        self.last_message = (None, None)

    def error(self, message):
        self.last_message = (message, logging.ERROR)

    def warn(self, message):
        self.last_message = (message, logging.WARNING)

    def info(self, message):
        self.last_message = (message, logging.INFO)

    def debug(self, message):
        self.last_message = (message, logging.DEBUG)

    def trace(self, message):
        self.last_message = (message, logging.NOTSET)


class TestPyLogging(unittest.TestCase):
    test_message = "This is a test message"
    test_format = "%(name)s %(levelname)s %(message)s"

    str_rep = {
        logging.ERROR: "ERROR",
        logging.WARNING: "WARNING",
        logging.INFO: "INFO",
        logging.DEBUG: "DEBUG",
        logging.NOTSET: "TRACE",
    }

    def setUp(self):
        self.library_logger = MockLibraryLogger()
        self.test_handler = RobotHandler(library_logger=self.library_logger)
        root = logging.getLogger()
        root.setLevel(logging.NOTSET)
        for handler in root.handlers:
            root.removeHandler(handler)
        root.addHandler(self.test_handler)

    def tearDown(self):
        root = logging.getLogger()
        root.removeHandler(self.test_handler)

    def test_python_logging_debug(self):
        logging.debug(self.test_message)
        self.assert_message(self.test_message, logging.DEBUG)

    def test_python_logging_info(self):
        logging.info(self.test_message)
        self.assert_message(self.test_message, logging.INFO)

    def test_python_logging_warn(self):
        logging.warning(self.test_message)
        self.assert_message(self.test_message, logging.WARNING)

    def test_python_logging_error(self):
        logging.error(self.test_message)
        self.assert_message(self.test_message, logging.ERROR)

    def test_python_logging_formatted_debug(self):
        old_formatter = self.test_handler.formatter
        formatter = logging.Formatter(fmt=self.test_format)
        self.test_handler.setFormatter(formatter)
        logging.debug(self.test_message)
        self.assert_formatted_message(logging.DEBUG)
        self.test_handler.setFormatter(old_formatter)

    def test_python_logging_formatted_info(self):
        old_formatter = self.test_handler.formatter
        formatter = logging.Formatter(fmt=self.test_format)
        self.test_handler.setFormatter(formatter)
        logging.info(self.test_message)
        self.assert_formatted_message(logging.INFO)
        self.test_handler.setFormatter(old_formatter)

    def test_python_logging_formatted_warn(self):
        old_formatter = self.test_handler.formatter
        formatter = logging.Formatter(fmt=self.test_format)
        self.test_handler.setFormatter(formatter)
        logging.warning(self.test_message)
        self.assert_formatted_message(logging.WARNING)
        self.test_handler.setFormatter(old_formatter)

    def test_python_logging_formatted_error(self):
        old_formatter = self.test_handler.formatter
        formatter = logging.Formatter(fmt=self.test_format)
        self.test_handler.setFormatter(formatter)
        logging.error(self.test_message)
        self.assert_formatted_message(logging.ERROR)
        self.test_handler.setFormatter(old_formatter)

    def assert_message(self, message, level):
        message_last, level_last = self.library_logger.last_message
        assert_equal(message_last, message)
        assert_equal(level_last, level)

    def assert_formatted_message(self, logging_level):
        formatted_message = f"root {self.str_rep[logging_level]} {self.test_message}"
        self.assert_message(formatted_message, logging_level)



from robot.api.deco import keyword, library

from CustomConverters import Number, string_to_int


@library(converters={Number: string_to_int})
class CustomConvertersWithLibraryDecorator:

    @keyword
    def using_library_decorator(self, value: Number, expected: int):
        assert value == expected

    @keyword(name='Embedded "${arg1}" should be equal to "${arg2}"')
    def embedded(self, value: Number, expected: int):
        assert value == expected



import unittest
import warnings
from pathlib import Path

from robot.utils.asserts import (assert_equal, assert_false, assert_not_equal, assert_raises,
                                 assert_raises_with_msg, assert_true)
from robot.model import TestSuite, TestCase, Keyword
from robot.model.testcase import TestCases


class TestTestCase(unittest.TestCase):

    def setUp(self):
        self.test = TestCase(tags=['t1', 't2'], name='test')

    def test_id_without_parent(self):
        assert_equal(self.test.id, 't1')

    def test_id_with_parent(self):
        suite = TestSuite()
        suite.suites.create().tests = [TestCase(), TestCase()]
        suite.suites.create().tests = [TestCase()]
        assert_equal(suite.suites[0].tests[0].id, 's1-s1-t1')
        assert_equal(suite.suites[0].tests[1].id, 's1-s1-t2')
        assert_equal(suite.suites[1].tests[0].id, 's1-s2-t1')

    def test_source(self):
        test = TestCase()
        assert_equal(test.source, None)
        suite = TestSuite()
        suite.tests.append(test)
        assert_equal(test.source, None)
        suite.tests.append(test)
        suite.source = '/unit/tests'
        assert_equal(test.source, Path('/unit/tests'))

    def test_setup(self):
        assert_equal(self.test.setup.__class__, Keyword)
        assert_equal(self.test.setup.name, None)
        assert_false(self.test.setup)
        self.test.setup.config(name='setup kw')
        assert_equal(self.test.setup.name, 'setup kw')
        assert_true(self.test.setup)
        self.test.setup = None
        assert_equal(self.test.setup.name, None)
        assert_false(self.test.setup)

    def test_teardown(self):
        assert_equal(self.test.teardown.__class__, Keyword)
        assert_equal(self.test.teardown.name, None)
        assert_false(self.test.teardown)
        self.test.teardown.config(name='teardown kw')
        assert_equal(self.test.teardown.name, 'teardown kw')
        assert_true(self.test.teardown)
        self.test.teardown = None
        assert_equal(self.test.teardown.name, None)
        assert_false(self.test.teardown)

    def test_modify_tags(self):
        self.test.tags.add(['t0', 't3'])
        self.test.tags.remove('T2')
        assert_equal(list(self.test.tags), ['t0', 't1', 't3'])

    def test_set_tags(self):
        self.test.tags = ['s2', 's1']
        self.test.tags.add('s3')
        assert_equal(list(self.test.tags), ['s1', 's2', 's3'])

    def test_longname(self):
        assert_equal(self.test.longname, 'test')
        self.test.parent = TestSuite(name='suite').suites.create(name='sub suite')
        assert_equal(self.test.longname, 'suite.sub suite.test')

    def test_slots(self):
        assert_raises(AttributeError, setattr, self.test, 'attr', 'value')

    def test_copy(self):
        test = self.test
        copy = test.copy()
        assert_equal(test.name, copy.name)
        copy.name += 'copy'
        assert_not_equal(test.name, copy.name)
        assert_equal(id(test.tags), id(copy.tags))

    def test_copy_with_attributes(self):
        test = TestCase(name='Orig', doc='Orig', tags=['orig'])
        copy = test.copy(name='New', doc='New', tags=['new'])
        assert_equal(copy.name, 'New')
        assert_equal(copy.doc, 'New')
        assert_equal(list(copy.tags), ['new'])

    def test_deepcopy_(self):
        test = self.test
        copy = test.deepcopy()
        assert_equal(test.name, copy.name)
        assert_not_equal(id(test.tags), id(copy.tags))

    def test_deepcopy_with_attributes(self):
        copy = TestCase(name='Orig').deepcopy(name='New', doc='New')
        assert_equal(copy.name, 'New')
        assert_equal(copy.doc, 'New')

    def test_keywords_deprecation(self):
        self.test.body = [Keyword(), Keyword(), Keyword()]
        with warnings.catch_warnings(record=True) as w:
            kws = self.test.keywords
            assert_equal(len(kws), 3)
            assert_true('deprecated' in str(w[0].message))
        assert_raises(AttributeError, kws.append, Keyword())
        assert_raises(AttributeError, setattr, self.test, 'keywords', [])


class TestStringRepresentation(unittest.TestCase):

    def setUp(self):
        self.empty = TestCase()
        self.ascii = TestCase(name='Kekkonen')
        self.non_ascii = TestCase(name=u'hyv\xe4 nimi')

    def test_str(self):
        for tc, expected in [(self.empty, ''),
                             (self.ascii, 'Kekkonen'),
                             (self.non_ascii, u'hyv\xe4 nimi')]:
            assert_equal(str(tc), expected)

    def test_repr(self):
        for tc, expected in [(self.empty, "TestCase(name='')"),
                             (self.ascii, "TestCase(name='Kekkonen')"),
                             (self.non_ascii, u"TestCase(name=%r)" % u'hyv\xe4 nimi')]:
            assert_equal(repr(tc), 'robot.model.' + expected)


class TestTestCases(unittest.TestCase):

    def setUp(self):
        self.suite = TestSuite()
        self.tests = TestCases(parent=self.suite,
                               tests=[TestCase(name=c) for c in 'abc'])

    def test_getitem_slice(self):
        tests = self.tests[:]
        assert_true(isinstance(tests, TestCases))
        assert_equal([t.name for t in tests], ['a', 'b', 'c'])
        tests.append(TestCase(name='d'))
        assert_equal([t.name for t in tests], ['a', 'b', 'c', 'd'])
        assert_true(all(t.parent is self.suite for t in tests))
        assert_equal([t.name for t in self.tests], ['a', 'b', 'c'])
        backwards = tests[::-1]
        assert_true(isinstance(tests, TestCases))
        assert_equal(list(backwards), list(reversed(tests)))

    def test_setitem_slice(self):
        tests = self.tests[:]
        tests[-1:] = [TestCase(name='b'), TestCase(name='a')]
        assert_equal([t.name for t in tests], ['a', 'b', 'b', 'a'])
        assert_true(all(t.parent is self.suite for t in tests))
        assert_raises_with_msg(TypeError,
                               'Only TestCase objects accepted, got TestSuite.',
                               tests.__setitem__, slice(0), [self.suite])


if __name__ == '__main__':
    unittest.main()



class Mandatory:

    def __init__(self, mandatory1, mandatory2):
        self.mandatory1 = mandatory1
        self.mandatory2 = mandatory2

    def get_args(self):
        return self.mandatory1, self.mandatory2


class Defaults:

    def __init__(self, mandatory, default1='value', default2=None):
        self.mandatory = mandatory
        self.default1 = default1
        self.default2 = default2

    def get_args(self):
        return self.mandatory, self.default1, self.default2


class Varargs(Mandatory):

    def __init__(self, mandatory, *varargs):
        Mandatory.__init__(self, mandatory, ' '.join(str(a) for a in varargs))


class Mixed(Defaults):

    def __init__(self, mandatory, default=42, *extra):
        Defaults.__init__(self, mandatory, default, 
                          ' '.join(str(a) for a in extra))



class DynamicLibraryWithoutGetKwArgsAndDoc:
    """Library doc set in class."""

    def __init__(self, doc=None):
        """Static __init__ doc."""
        if doc:
            self.__doc__ = doc

    def get_keyword_names(self):
        return ['Keyword']

    def run_keyword(self, name, args):
        pass



from io import StringIO
import unittest

from robot.output import LOGGER
from robot.reporting.resultwriter import ResultWriter, Results
from robot.result.executionerrors import ExecutionErrors
from robot.result import TestSuite, Result
from robot.utils.asserts import assert_true, assert_equal


LOGGER.unregister_console_logger()


class TestReporting(unittest.TestCase):
    EXPECTED_SUITE_NAME = 'My Suite Name'
    EXPECTED_TEST_NAME = 'My Test Name'
    EXPECTED_KEYWORD_NAME = 'My Keyword Name'
    EXPECTED_FAILING_TEST = 'My Failing Test'
    EXPECTED_DEBUG_MESSAGE = '1111DEBUG777'
    EXPECTED_ERROR_MESSAGE = 'ERROR M355463'

    def test_only_output(self):
        output = ClosableOutput('output.xml')
        self._write_results(output=output)
        self._verify_output(output.value)

    def test_only_xunit(self):
        xunit = ClosableOutput('xunit.xml')
        self._write_results(xunit=xunit)
        self._verify_xunit(xunit.value)

    def test_only_log(self):
        log = ClosableOutput('log.html')
        self._write_results(log=log)
        self._verify_log(log.value)

    def test_only_report(self):
        report = ClosableOutput('report.html')
        self._write_results(report=report)
        self._verify_report(report.value)

    def test_log_and_report(self):
        log = ClosableOutput('log.html')
        report = ClosableOutput('report.html')
        self._write_results(log=log, report=report)
        self._verify_log(log.value)
        self._verify_report(report.value)

    def test_generate_all(self):
        output = ClosableOutput('o.xml')
        xunit = ClosableOutput('x.xml')
        log = ClosableOutput('l.html')
        report = ClosableOutput('r.html')
        self._write_results(output=output, xunit=xunit, log=log, report=report)
        self._verify_output(output.value)
        self._verify_xunit(xunit.value)
        self._verify_log(log.value)
        self._verify_report(report.value)

    def test_js_generation_does_not_prune_given_result(self):
        result = self._get_execution_result()
        results = Results(StubSettings(), result)
        _ = results.js_result
        for test in results.result.suite.tests:
            assert_true(len(test.body) > 0)

    def test_js_generation_prunes_read_result(self):
        result = self._get_execution_result()
        results = Results(StubSettings(), 'output.xml')
        assert_equal(results._result, None)
        results._result = result  # Fake reading results
        _ = results.js_result
        for test in result.suite.tests:
            assert_equal(len(test.body), 0)

    def _write_results(self, **settings):
        result = self._get_execution_result()
        settings = StubSettings(**settings)
        rc = ResultWriter(result).write_results(settings)
        assert_equal(rc, 1)

    def _get_execution_result(self):
        suite = TestSuite(name=self.EXPECTED_SUITE_NAME)
        tc = suite.tests.create(name=self.EXPECTED_TEST_NAME, status='PASS')
        tc.body.create_keyword(kwname=self.EXPECTED_KEYWORD_NAME, status='PASS')
        tc = suite.tests.create(name=self.EXPECTED_FAILING_TEST)
        kw = tc.body.create_keyword(kwname=self.EXPECTED_KEYWORD_NAME)
        kw.body.create_message(message=self.EXPECTED_DEBUG_MESSAGE,
                               level='DEBUG', timestamp='20201212 12:12:12.000')
        errors = ExecutionErrors()
        errors.messages.create(message=self.EXPECTED_ERROR_MESSAGE,
                               level='ERROR', timestamp='20201212 12:12:12.000')
        return Result(root_suite=suite, errors=errors)

    def _verify_output(self, content):
        assert_true(self.EXPECTED_SUITE_NAME in content)
        assert_true(self.EXPECTED_TEST_NAME in content)
        assert_true(self.EXPECTED_FAILING_TEST in content)
        assert_true(self.EXPECTED_KEYWORD_NAME in content)
        assert_true(self.EXPECTED_DEBUG_MESSAGE in content)
        assert_true(self.EXPECTED_ERROR_MESSAGE in content)

    def _verify_xunit(self, content):
        assert_true(self.EXPECTED_SUITE_NAME in content)
        assert_true(self.EXPECTED_TEST_NAME in content)
        assert_true(self.EXPECTED_FAILING_TEST in content)
        assert_true(self.EXPECTED_KEYWORD_NAME not in content)
        assert_true(self.EXPECTED_DEBUG_MESSAGE not in content)
        assert_true(self.EXPECTED_ERROR_MESSAGE not in content)

    def _verify_log(self, content):
        self._verify_output(content)

    def _verify_report(self, content):
        assert_true(self.EXPECTED_SUITE_NAME in content)
        assert_true(self.EXPECTED_TEST_NAME in content)
        assert_true(self.EXPECTED_FAILING_TEST in content)
        assert_true(self.EXPECTED_KEYWORD_NAME not in content)
        assert_true(self.EXPECTED_DEBUG_MESSAGE not in content)
        assert_true(self.EXPECTED_ERROR_MESSAGE not in content)


class StubSettings:
    log = None
    log_config = {}
    split_log = False
    report = None
    report_config = None
    output = None
    xunit = None
    status_rc = True
    suite_config = {}
    statistics_config = {}
    xunit_skip_noncritical = False
    expand_keywords = None

    def __init__(self, **settings):
        self.__dict__.update(settings)


class ClosableOutput:

    def __init__(self, path):
        self._output = StringIO()
        self._path = path

    def __enter__(self):
        return self

    def __exit__(self, *args):
        self.close()

    def write(self, data):
        self._output.write(data)

    def close(self):
        self.value = self._output.getvalue()
        self._output.close()

    def __str__(self):
        return self._path


if __name__ == '__main__':
    unittest.main()



ROBOT_LISTENER_API_VERSION = 2


def startTest(name, info):
    print('[START] [original] %s [resolved] %s' % (info['originalname'], name))


def end_test(name, info):
    print('[END] [original] %s [resolved] %s' % (info['originalname'], name))



import time


class FailUntilSucceeds:
    ROBOT_LIBRARY_SCOPE = 'TESTCASE'

    def __init__(self, times_to_fail=0):
        self.times_to_fail = int(times_to_fail)

    def set_times_to_fail(self, times_to_fail):
        self.__init__(times_to_fail)

    def fail_until_retried_often_enough(self, message="Hello", sleep=0):
        self.times_to_fail -= 1
        time.sleep(sleep)
        if self.times_to_fail >= 0:
            raise Exception('Still %d times to fail!' % self.times_to_fail)
        return message



from robot.model import SuiteVisitor


class ModelModifier(SuiteVisitor):

    def __init__(self, *tags, **extra):
        if extra:
            tags += tuple('%s-%s' % item for item in extra.items())
        self.config = tags or ('visited',)

    def start_suite(self, suite):
        config = self.config
        if config[0] == 'FAIL':
            raise RuntimeError(' '.join(self.config[1:]))
        elif config[0] == 'CREATE':
            tc = suite.tests.create(**dict(conf.split('-', 1) for conf in config[1:]))
            tc.body.create_keyword('No operation')
            self.config = []
        elif config == ('REMOVE', 'ALL', 'TESTS'):
            suite.tests = []
        else:
            suite.tests = [t for t in suite.tests if not t.tags.match('fail')]

    def start_test(self, test):
        test.tags.add(self.config)

    def start_for(self, for_):
        if for_.parent.name == 'FOR IN RANGE':
            for_.flavor = 'IN'
            for_.values = ['FOR', 'is', 'modified!']

    def start_for_iteration(self, iteration):
        for name, value in iteration.variables.items():
            iteration.variables[name] = value + ' (modified)'
        iteration.variables['${x}'] = 'new'

    def start_if_branch(self, branch):
        if branch.condition == "'IF' == 'WRONG'":
            branch.condition = 'True'
            # With Robot
            if not hasattr(branch, 'status'):
                branch.body[0].config(name='Log', args=['going here!'])
            # With Rebot
            elif branch.status == 'NOT RUN':
                branch.status = 'PASS'
                branch.condition = 'modified'
                branch.body[0].args = ['got here!']
        if branch.condition == '${i} == 9':
            branch.condition = 'False'



from robot.libraries.BuiltIn import BuiltIn

BIN = BuiltIn()
ROBOT_LISTENER_API_VERSION = 2


def start_keyword(*args):
    if BIN.get_variables()['${TESTNAME}'] == 'Listener Using BuiltIn':
        BIN.set_test_variable('${SET BY LISTENER}', 'quux')



#!/usr/bin/env python

import sys

stdout = sys.argv[1] if len(sys.argv) > 1 else 'stdout'
stderr = sys.argv[2] if len(sys.argv) > 2 else 'stderr'
rc = int(sys.argv[3]) if len(sys.argv) > 3 else 0

sys.stdout.write(stdout + '\n')
sys.stderr.write(stderr + '\n')
sys.exit(rc)



#!/usr/bin/env python3

"""Libdoc JSON schema model definition.

The schema is modeled using pydantic in this file. After updating the model,
execute this file to regenerate the actual schema file in libdoc.json.

https://pydantic-docs.helpmanual.io/usage/schema/
"""

from datetime import datetime
from enum import Enum
from pathlib import Path
from typing import List, Optional, Union

from pydantic import BaseModel as PydanticBaseModel, Field, PositiveInt


class BaseModel(PydanticBaseModel):

    # Workaround for Pydantic not supporting nullable types.
    # https://github.com/pydantic/pydantic/issues/1270#issuecomment-729555558
    class Config:
        @staticmethod
        def schema_extra(schema, model):
            for prop, value in schema.get('properties', {}).items():
                # retrieve right field from alias or name
                field = [x for x in model.__fields__.values() if x.alias == prop][0]
                if field.allow_none:
                    # only one type e.g. {'type': 'integer'}
                    if 'type' in value:
                        value['anyOf'] = [{'type': value.pop('type')}]
                    # only one $ref e.g. from other model
                    elif '$ref' in value:
                        if issubclass(field.type_, PydanticBaseModel):
                            # add 'title' in schema to have the exact same behaviour as the rest
                            value['title'] = field.type_.__config__.title or field.type_.__name__
                        value['anyOf'] = [{'$ref': value.pop('$ref')}]
                    value['anyOf'].append({'type': 'null'})


class SpecVersion(int, Enum):
    """Version of the spec."""
    VERSION = 2


class DocumentationType(str, Enum):
    """Type of the doc: LIBRARY or RESOURCE."""
    LIBRARY = 'LIBRARY'
    RESOURCE = 'RESOURCE'
    SUITE = 'SUITE'


class LibraryScope(str, Enum):
    "Library scope: GLOBAL, SUITE or TEST."
    GLOBAL = 'GLOBAL'
    SUITE = 'SUITE'
    TEST = 'TEST'


class DocumentationFormat(str, Enum):
    """Documentation format, typically HTML."""
    ROBOT = 'ROBOT'
    HTML = 'HTML'
    TEXT = 'TEXT'
    REST = 'REST'


class ArgumentKind(str, Enum):
    """Argument kind: positional, named, vararg, etc."""
    POSITIONAL_ONLY = 'POSITIONAL_ONLY'
    POSITIONAL_ONLY_MARKER = 'POSITIONAL_ONLY_MARKER'
    POSITIONAL_OR_NAMED = 'POSITIONAL_OR_NAMED'
    VAR_POSITIONAL = 'VAR_POSITIONAL'
    NAMED_ONLY_MARKER = 'NAMED_ONLY_MARKER'
    NAMED_ONLY = 'NAMED_ONLY'
    VAR_NAMED = 'VAR_NAMED'


class ArgumentType(BaseModel):
    name: str
    typedoc: Union[str, None] = Field(description="Map type to info in 'typedocs'.")
    nested: List['ArgumentType']
    union: bool


class Argument(BaseModel):
    """Keyword argument."""
    name: str
    type: Union[ArgumentType, None]
    types: List[str] = Field(description="Deprecated. Use 'type' instead.")
    typedocs: dict = Field(description="Deprecated. Use 'type' instead.")
    defaultValue: Union[str, None] = Field(description="Possible default value or 'null'.")
    kind: ArgumentKind
    required: bool
    repr: str


class Keyword(BaseModel):
    name: str
    args: List[Argument]
    doc: str
    shortdoc: str
    tags: List[str]
    private: Optional[bool]
    deprecated: Optional[bool]
    source: Path
    lineno: int


class TypeDocType(str, Enum):
    """Type of the type: Standard, Enum, TypedDict or Custom."""
    Standard = 'Standard'
    Enum = 'Enum'
    TypedDict = 'TypedDict'
    Custom = 'Custom'


class EnumMember(BaseModel):
    name: str
    value: str


class TypedDictItem(BaseModel):
    key: str
    type: str
    required: Union[bool, None]    # This is overridden below.


class TypeDoc(BaseModel):
    type: TypeDocType
    name: str
    doc: str
    usages: List[str] = Field(description='List of keywords using this type.')
    accepts: List[str] = Field(description='List of accepted argument types.')
    members: Optional[List[EnumMember]] = Field(description='Used only with Enum type.')
    items: Optional[List[TypedDictItem]] = Field(description='Used only with TypedDict type.')


class Libdoc(BaseModel):
    """Libdoc JSON spec file schema.

    Compatible with JSON Schema Draft 2020-12.
    """
    specversion: SpecVersion
    name: str
    doc: str
    version: str
    generated: datetime
    type: DocumentationType
    scope: LibraryScope
    docFormat: DocumentationFormat
    source: Path
    lineno: PositiveInt
    tags: List[str] = Field(description='List of all tags used by keywords.')
    inits: List[Keyword]
    keywords: List[Keyword]
    dataTypes: dict = Field({}, description="Deprecated. Use 'typedocs' instead.")
    typedocs: List[TypeDoc]

    # pydantic doesn't add schema version automatically.
    # https://github.com/samuelcolvin/pydantic/issues/1478
    class Config:
        schema_extra = {
            '$schema': 'https://json-schema.org/draft/2020-12/schema'
        }


if __name__ == '__main__':
    path = Path(__file__).parent / 'libdoc.json'
    with open(path, 'w') as f:
        f.write(Libdoc.schema_json(indent=2))
    print(path.absolute())



import sys


def print_one_html_line():
    print('*HTML* <a href="http://www.google.com">Google</a>')


def print_many_html_lines():
    print('*HTML* <table border=1>\n<tr><td>0,0</td><td>0,1</td></tr>')
    print('<tr><td>1,0</td><td>1,1</td></tr>\n</table>')
    print('*HTML*This is html <hr>')
    print('*INFO*This is not html <br>')


def print_html_to_stderr():
    print('*HTML* <i>Hello, stderr!!</i>', file=sys.stderr)


def print_console():
    print('*CONSOLE* Hello info and console!')


def print_with_all_levels():
    for level in 'TRACE DEBUG INFO CONSOLE HTML WARN ERROR'.split():
        print('*%s* %s message' % (level, level.title()))



import os
from stat import S_IREAD, S_IWRITE, S_IEXEC

from robot.api import logger


class TestHelper:

    def set_read_only(self, path):
        os.chmod(path, S_IREAD)

    def set_read_write(self, path):
        os.chmod(path, S_IREAD | S_IWRITE)

    def set_read_write_execute(self, path):
        os.chmod(path, S_IREAD | S_IWRITE | S_IEXEC)

    def remove_permissions(self, path):
        os.chmod(path, 0)

    def file_should_have_correct_line_separators(self, output, sep=os.linesep):
        if os.path.isfile(output):
            with open(output, 'rb') as infile:
                output = infile.read().decode('UTF-8')
        if sep not in output:
            self._wrong_separators('Output has no %r separators' % sep, output)
        extra_r = output.replace(sep, '').count('\r')
        extra_n = output.replace(sep, '').count('\n')
        if extra_r or extra_n:
            self._wrong_separators("Output has %d extra \\r and %d extra \\n"
                                   % (extra_r, extra_n), output)

    def _wrong_separators(self, message, output):
        logger.info(repr(output).replace('\\n', '\\n\n'))
        failure = AssertionError(message)
        failure.ROBOT_CONTINUE_ON_FAILURE = True
        raise failure



import unittest

from robot.model import Body, BodyItem, If, For, Keyword, TestCase, TestSuite
from robot.result.model import Body as ResultBody
from robot.utils.asserts import assert_equal, assert_raises_with_msg


class TestBody(unittest.TestCase):

    def test_no_create(self):
        error = ("'robot.model.Body' object has no attribute 'create'. "
                 "Use item specific methods like 'create_keyword' instead.")
        assert_raises_with_msg(AttributeError, error,
                               getattr, Body(), 'create')
        assert_raises_with_msg(AttributeError, error.replace('.model.', '.result.'),
                               getattr, ResultBody(), 'create')

    def test_base_body_does_not_support_filtering_by_messages(self):
        error = "'robot.model.Body' object does not support filtering by 'messages'."
        assert_raises_with_msg(TypeError, error,
                               Body().filter, messages=True)
        assert_raises_with_msg(TypeError, error,
                               Body().filter, messages=False)

    def test_filter_when_messages_are_supported(self):
        body = ResultBody()
        k1 = body.create_keyword()
        m1 = body.create_message()
        i1 = body.create_if()
        f1 = body.create_for()
        i2 = body.create_if()
        k2 = body.create_keyword()
        m2 = body.create_message()
        m3 = body.create_message()
        assert_equal(body.filter(keywords=True), [k1, k2])
        assert_equal(body.filter(keywords=False), [m1, i1, f1, i2, m2, m3])
        assert_equal(body.filter(messages=True), [m1, m2, m3])
        assert_equal(body.filter(messages=False), [k1, i1, f1, i2, k2])
        assert_equal(body.filter(keywords=True, messages=True), [k1, m1, k2, m2, m3])
        assert_equal(body.filter(keywords=False, messages=False), [i1, f1, i2])
        assert_equal(body.filter(), [k1, m1, i1, f1, i2, k2, m2, m3])

    def test_cannot_filter_with_both_includes_and_excludes(self):
        assert_raises_with_msg(
            ValueError,
            'Items cannot be both included and excluded by type.',
            ResultBody().filter, keywords=True, messages=False
        )

    def test_filter_with_predicate(self):
        x = Keyword(name='x')
        predicate = lambda item: item.name == 'x'
        body = Body(items=[Keyword(), x, Keyword()])
        assert_equal(body.filter(predicate=predicate), [x])
        body = Body(items=[Keyword(), If(), x, For(), Keyword()])
        assert_equal(body.filter(keywords=True, predicate=predicate), [x])


class TestBodyItem(unittest.TestCase):

    def test_id_without_parent(self):
        item = BodyItem()
        item.parent = None
        assert_equal(item.id, 'k1')

    def test_id_with_parent(self):
        tc = TestCase()
        tc.body = [BodyItem(), BodyItem(), BodyItem()]
        assert_equal([item.id for item in tc.body], ['t1-k1', 't1-k2', 't1-k3'])

    def test_id_with_parent_having_setup_and_teardown(self):
        tc = TestCase()
        assert_equal(tc.setup.config(name='S').id, 't1-k1')
        assert_equal(tc.teardown.config(name='T').id, 't1-k2')
        tc.body = [BodyItem(), BodyItem(), BodyItem()]
        assert_equal([item.id for item in tc.body], ['t1-k2', 't1-k3', 't1-k4'])
        assert_equal(tc.setup.id, 't1-k1')
        assert_equal(tc.teardown.id, 't1-k5')

    def test_id_when_item_not_in_parent(self):
        tc = TestCase(parent=TestSuite(parent=TestSuite()))
        assert_equal(tc.id, 's1-s1-t1')
        assert_equal(Keyword(parent=tc).id, 's1-s1-t1-k1')
        tc.body.create_keyword()
        tc.body.create_if().body.create_branch()
        assert_equal(Keyword(parent=tc).id, 's1-s1-t1-k3')

    def test_id_with_if(self):
        tc = TestCase()
        root = tc.body.create_if()
        assert_equal(root.id, None)
        branch = root.body.create_branch()
        assert_equal(branch.id, 't1-k1')
        assert_equal(branch.body.create_keyword().id, 't1-k1-k1')
        assert_equal(branch.body.create_keyword().id, 't1-k1-k2')
        branch = root.body.create_branch()
        assert_equal(branch.id, 't1-k2')
        assert_equal(branch.body.create_keyword().id, 't1-k2-k1')
        assert_equal(branch.body.create_keyword().id, 't1-k2-k2')
        assert_equal(tc.body.create_keyword().id, 't1-k3')

    def test_id_with_try(self):
        tc = TestCase()
        root = tc.body.create_try()
        assert_equal(root.id, None)
        branch = root.body.create_branch()
        assert_equal(branch.id, 't1-k1')
        assert_equal(branch.body.create_keyword().id, 't1-k1-k1')
        assert_equal(branch.body.create_keyword().id, 't1-k1-k2')
        branch = root.body.create_branch()
        assert_equal(branch.id, 't1-k2')
        assert_equal(branch.body.create_keyword().id, 't1-k2-k1')
        assert_equal(branch.body.create_keyword().id, 't1-k2-k2')
        assert_equal(tc.body.create_keyword().id, 't1-k3')

    def test_id_with_if_and_try(self):
        tc = TestCase()
        # IF
        root = tc.body.create_if()
        assert_equal(root.id, None)
        branch = root.body.create_branch()
        assert_equal(branch.id, 't1-k1')
        assert_equal(branch.body.create_keyword().id, 't1-k1-k1')
        assert_equal(branch.body.create_keyword().id, 't1-k1-k2')
        branch = root.body.create_branch()
        assert_equal(branch.id, 't1-k2')
        assert_equal(branch.body.create_keyword().id, 't1-k2-k1')
        assert_equal(branch.body.create_keyword().id, 't1-k2-k2')
        assert_equal(tc.body.create_keyword().id, 't1-k3')
        # TRY
        root = tc.body.create_try()
        assert_equal(root.id, None)
        branch = root.body.create_branch()
        assert_equal(branch.id, 't1-k4')
        assert_equal(branch.body.create_keyword().id, 't1-k4-k1')
        assert_equal(branch.body.create_keyword().id, 't1-k4-k2')
        branch = root.body.create_branch()
        assert_equal(branch.id, 't1-k5')
        assert_equal(branch.body.create_keyword().id, 't1-k5-k1')
        assert_equal(branch.body.create_keyword().id, 't1-k5-k2')
        assert_equal(tc.body.create_keyword().id, 't1-k6')
        # IF again
        root = tc.body.create_if()
        assert_equal(root.id, None)
        branch = root.body.create_branch()
        assert_equal(branch.id, 't1-k7')
        assert_equal(branch.body.create_keyword().id, 't1-k7-k1')
        assert_equal(branch.body.create_keyword().id, 't1-k7-k2')
        branch = root.body.create_branch()
        assert_equal(branch.id, 't1-k8')
        assert_equal(branch.body.create_keyword().id, 't1-k8-k1')
        assert_equal(branch.body.create_keyword().id, 't1-k8-k2')
        assert_equal(tc.body.create_keyword().id, 't1-k9')


if __name__ == '__main__':
    unittest.main()



import sys
from xmlrpc.server import SimpleXMLRPCServer

from remoteserver import announce_port


class SimpleServer(SimpleXMLRPCServer):

    def __init__(self, port=8270, port_file=None):
        SimpleXMLRPCServer.__init__(self, ('127.0.0.1', int(port)))
        self.register_function(self.get_keyword_names)
        self.register_function(self.run_keyword)
        announce_port(self.socket, port_file)
        self.serve_forever()

    def get_keyword_names(self):
        return ['Passing', 'Failing', 'Traceback', 'Returning', 'Logging',
                'Extra stuff in result dictionary',
                'Conflict', 'Should Be True']

    def run_keyword(self, name, args):
        if name == 'Passing':
            return {'status': 'PASS'}
        if name == 'Failing':
            return {'status': 'FAIL', 'error': ' '.join(args)}
        if name == 'Traceback':
            return {'status': 'FAIL', 'traceback': ' '.join(args)}
        if name == 'Returning':
            return {'status': 'PASS', 'return': ' '.join(args)}
        if name == 'Logging':
            return {'status': 'PASS', 'output': '\n'.join(args)}
        if name == 'Extra stuff in result dictionary':
            return {'status': 'PASS', 'extra': 'stuff', 'is': 'ignored'}
        if name == 'Conflict':
            return {'status': 'FAIL', 'error': 'Should not be executed'}
        if name == 'Should Be True':
            return {'status': 'PASS', 'output': 'Always passes'}


if __name__ == '__main__':
    SimpleServer(*sys.argv[1:])



from decimal import Decimal

from robot.api.deco import keyword


class Dynamic:

    def get_keyword_names(self):
        return [name for name in dir(self)
                if hasattr(getattr(self, name), 'robot_name')]

    def run_keyword(self, name, args, kwargs):
        return getattr(self, name)(*args, **kwargs)

    def get_keyword_arguments(self, name):
        if name == 'default_values':
            return [('first', 1), ('first_expected', 1),
                    ('middle', None), ('middle_expected', None),
                    ('last', True), ('last_expected', True)]
        if name == 'kwonly_defaults':
            return [('*',), ('first', 1), ('first_expected', 1),
                    ('last', True), ('last_expected', True)]
        if name == 'default_values_when_types_are_none':
            return [('value', True), ('expected', None)]
        return ['value', 'expected=None']

    def get_keyword_types(self, name):
        return getattr(self, name).robot_types

    @keyword(types=[int])
    def list_of_types(self, value, expected=None):
        self._validate_type(value, expected)

    @keyword(types={'value': Decimal})
    def dict_of_types(self, value, expected=None):
        self._validate_type(value, expected)

    @keyword(types=['bytes'])
    def list_of_aliases(self, value, expected=None):
        self._validate_type(value, expected)

    @keyword(types={'value': 'Dictionary'})
    def dict_of_aliases(self, value, expected=None):
        self._validate_type(value, expected)

    @keyword
    def default_values(self, first=1, first_expected=1,
                       middle=None, middle_expected=None,
                       last=True, last_expected=True):
        self._validate_type(first, first_expected)
        self._validate_type(middle, middle_expected)
        self._validate_type(last, last_expected)

    @keyword
    def kwonly_defaults(self, first=1, first_expected=1,
                        last=True, last_expected=True):
        self._validate_type(first, first_expected)
        self._validate_type(last, last_expected)

    @keyword(types=None)
    def default_values_when_types_are_none(self, value=True, expected=None):
        self._validate_type(value, expected)

    def _validate_type(self, argument, expected):
        if isinstance(expected, str):
            expected = eval(expected)
        if argument != expected or type(argument) != type(expected):
            raise AssertionError('%r (%s) != %r (%s)'
                                 % (argument, type(argument).__name__,
                                    expected, type(expected).__name__))
